@@ -32,7 +32,6 @@ public class CachingClassLoader extends ClassLoader implements ClassLoaderHierar
 
     static {
         try {
-            //noinspection Since15
             ClassLoader.registerAsParallelCapable();
         } catch (NoSuchMethodError ignore) {
             // Not supported on Java 6
@@ -78,6 +77,7 @@ public class CachingClassLoader extends ClassLoader implements ClassLoaderHierar
         return result;
     }
 
+    @Override
     public void visit(ClassLoaderVisitor visitor) {
         visitor.visitSpec(new Spec());
         visitor.visitParent(getParent());
@@ -89,6 +89,11 @@ public class CachingClassLoader extends ClassLoader implements ClassLoaderHierar
         resources.clear();
     }
 
+    @Override
+    public String toString() {
+        return CachingClassLoader.class.getSimpleName() + "(" + getParent() + ")";
+    }
+
     public static class Spec extends ClassLoaderSpec {
         @Override
         public boolean equals(Object obj) {
