@@ -17,7 +17,9 @@
 package org.gradle.internal.operations;
 
 /**
- * Represents a build operation that is also runnable and therefore does not require a {@link BuildOperationWorker}.
+ * Represents a build operation that is directly runnable and therefore does not require a {@link BuildOperationWorker}.
  */
-public interface RunnableBuildOperation extends BuildOperation, Runnable {
+public interface RunnableBuildOperation extends BuildOperation {
+
+    void run(BuildOperationContext context);
 }
