@@ -18,17 +18,16 @@ package org.gradle.internal.service;
 import org.gradle.api.Action;
 import org.gradle.internal.Factory;
 import org.gradle.internal.concurrent.CompositeStoppable;
-import org.gradle.internal.concurrent.Stoppable;
 
 import java.io.Closeable;
 import java.lang.annotation.Annotation;
 import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Member;
+import java.lang.reflect.Modifier;
 import java.lang.reflect.ParameterizedType;
 import java.lang.reflect.Type;
 import java.lang.reflect.WildcardType;
-import java.util.AbstractList;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
@@ -54,7 +53,7 @@ import java.util.concurrent.atomic.AtomicReference;
  *
  * <li>Adding a factory method. A factory method should have a name that starts with 'create', and have a non-void return type. For example, <code>protected SomeService createSomeService() { ....
  * }</code>. Parameters are injected using services from this registry or its parents. Parameter of type {@link ServiceRegistry} will receive the service registry that owns the service. Parameter of
- * type {@code List<T>} will receive all services of type T, if any. If a parameter has the same type as the return type of the factory method, then that parameter will be looked up in the parent registry.
+ * type {@code List<T>} will receive all services of type T, if any. If a parameter has the same type as the return type of the factory method, then that parameter will be located in the parent registry.
  * This allows decorating services.</li>
  *
  * <li>Adding a configure method. A configure method should be called 'configure', take a {@link ServiceRegistration} parameter, and a have a void return type. Additional parameters are injected using
@@ -69,8 +68,8 @@ import java.util.concurrent.atomic.AtomicReference;
  *
  * <p>Service registries are arranged in a hierarchy. If a service of a given type cannot be located, the registry uses its parent registry, if any, to locate the service.</p>
  */
-public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
-    private enum State {INIT, STARTED, CLOSED};
+public class DefaultServiceRegistry implements ServiceRegistry, Closeable, ContainsServices {
+    private enum State {INIT, STARTED, CLOSED}
     private final static ServiceRegistry[] NO_PARENTS = new ServiceRegistry[0];
     private final static Service[] NO_DEPENDENTS = new Service[0];
     private final static Object[] NO_PARAMS = new Object[0];
@@ -98,7 +97,6 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
     public DefaultServiceRegistry(String displayName, ServiceRegistry... parents) {
         this.displayName = displayName;
         this.ownServices = new OwnServices();
-        this.thisAsServiceProvider = new ParentServices(this);
         if (parents.length == 0) {
             this.parentServices = null;
             this.allServices = ownServices;
@@ -106,6 +104,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
             parentServices = setupParentServices(parents);
             allServices = new CompositeServiceProvider(ownServices, parentServices);
         }
+        this.thisAsServiceProvider = allServices;
 
         findProviderMethods(this);
     }
@@ -124,15 +123,16 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         return parentServices;
     }
 
-    private ServiceProvider asProvider() {
+    @Override
+    public ServiceProvider asProvider() {
         return thisAsServiceProvider;
     }
 
     private static ServiceProvider toParentServices(ServiceRegistry serviceRegistry) {
-        if (serviceRegistry instanceof DefaultServiceRegistry) {
-            return ((DefaultServiceRegistry) serviceRegistry).asProvider();
+        if (serviceRegistry instanceof ContainsServices) {
+            return new ParentServices(((ContainsServices) serviceRegistry).asProvider());
         }
-        return new ParentServices(serviceRegistry);
+        throw new IllegalArgumentException(String.format("Service registry %s cannot be used as a parent for another service registry.", serviceRegistry));
     }
 
     /**
@@ -214,14 +214,17 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
 
     private ServiceRegistration newRegistration() {
         return new ServiceRegistration() {
+            @Override
             public <T> void add(Class<T> serviceType, T serviceInstance) {
                 DefaultServiceRegistry.this.add(serviceType, serviceInstance);
             }
 
+            @Override
             public void add(Class<?> serviceType) {
                 ownServices.add(new ConstructorService(DefaultServiceRegistry.this, serviceType));
             }
 
+            @Override
             public void addProvider(Object provider) {
                 DefaultServiceRegistry.this.addProvider(provider);
             }
@@ -249,6 +252,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
     /**
      * Closes all services for this registry. For each service, if the service has a public void close() or stop() method, that method is called to close the service.
      */
+    @Override
     public void close() {
         noLongerMutable();
         if (state.compareAndSet(State.STARTED, State.CLOSED)) {
@@ -273,6 +277,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         return state.get() == State.CLOSED;
     }
 
+    @Override
     public <T> T get(Class<T> serviceType) throws UnknownServiceException, ServiceLookupException {
         return serviceType.cast(get((Type) serviceType));
     }
@@ -323,81 +328,53 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
     public <T> List<T> getAll(Class<T> serviceType) throws ServiceLookupException {
         assertValidServiceType(serviceType);
         List<T> services = new ArrayList<T>();
-        collectServices(serviceType, new InstanceUnpackingList<T>(serviceType, services));
+        serviceRequested();
+        allServices.getAll(serviceType, new InstanceUnpackingVisitor<T>(serviceType, services));
         return services;
     }
 
-    private <T> void collectServices(Class<T> serviceType, List<Service> results) {
-        serviceRequested();
-        allServices.getAll(serviceType, results);
-    }
-
-    private static class InstanceUnpackingList<T> extends AbstractList<Service> {
+    private static class InstanceUnpackingVisitor<T> implements ServiceProvider.Visitor {
         private final Class<T> serviceType;
         private final List<T> delegate;
 
-        private InstanceUnpackingList(Class<T> serviceType, List<T> delegate) {
+        private InstanceUnpackingVisitor(Class<T> serviceType, List<T> delegate) {
             this.serviceType = serviceType;
             this.delegate = delegate;
         }
 
         @Override
-        public Service get(int index) {
-            throw new UnsupportedOperationException();
-        }
-
-        @Override
-        public boolean add(Service provider) {
-            return delegate.add(serviceType.cast(provider.get()));
-        }
-
-        @Override
-        public int size() {
-            return delegate.size();
+        public void visit(Service service) {
+            delegate.add(serviceType.cast(service.get()));
         }
     }
 
+    private static class CollectingVisitor implements ServiceProvider.Visitor {
+        private final List<Service> delegate;
+
+        private CollectingVisitor(List<Service> delegate) {
+            this.delegate = delegate;
+        }
+
+        @Override
+        public void visit(Service service) {
+            delegate.add(service);
+        }
+    }
+
+    @Override
     public <T> T newInstance(Class<T> type) {
         return getFactory(type).create();
     }
 
-    /**
-     * Wraps a single service instance. Implementations must be thread safe.
-     */
-    interface Service {
-        String getDisplayName();
-
-        Object get();
-
-        void requiredBy(ServiceProvider serviceProvider);
-    }
-
-    /**
-     * Provides a set of zero or more services. The get-methods may be called concurrently. {@link #stop()} is guaranteed to be only called once,
-     * after all get-methods have completed.
-     */
-    interface ServiceProvider extends Stoppable {
-        /**
-         * Locates a service instance of the given type. Returns null if this provider does not provide a service of this type.
-         */
-        Service getService(Type serviceType);
-
-        /**
-         * Locates a factory for services of the given type. Returns null if this provider does not provide any services of this type.
-         */
-        Service getFactory(Class<?> type);
-
-        /**
-         * Collects all services of the given type.
-         */
-        void getAll(Class<?> serviceType, List<Service> result);
-    }
-
     private class OwnServices implements ServiceProvider {
         private final Map<Class<?>, List<ServiceProvider>> providersByType = new HashMap<Class<?>, List<ServiceProvider>>(16, 0.5f);
         private final CompositeStoppable stoppable = CompositeStoppable.stoppable();
         private ProviderAnalyser analyser = new ProviderAnalyser();
 
+        public OwnServices() {
+            providersByType.put(ServiceRegistry.class, Collections.<ServiceProvider>singletonList(new ThisAsService()));
+        }
+
         @Override
         public Service getFactory(Class<?> type) {
             List<ServiceProvider> serviceProviders = getProviders(Factory.class);
@@ -481,10 +458,11 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         }
 
         @Override
-        public void getAll(Class<?> serviceType, List<Service> result) {
+        public Visitor getAll(Class<?> serviceType, ServiceProvider.Visitor visitor) {
             for (ServiceProvider serviceProvider : getProviders(serviceType)) {
-                serviceProvider.getAll(serviceType, result);
+                visitor = serviceProvider.getAll(serviceType, visitor);
             }
+            return visitor;
         }
 
         @Override
@@ -518,6 +496,10 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
                     return;
                 }
                 if (seen.add(type)) {
+                    if (type.equals(ServiceRegistry.class)) {
+                        // Disallow custom services of type ServiceRegistry, as these are automatically provided
+                        throw new IllegalArgumentException("Cannot define a service of type ServiceRegistry: " + serviceProvider);
+                    }
                     putServiceType(type, serviceProvider);
                     analyseType(type.getSuperclass(), serviceProvider);
                     for (Class<?> iface : type.getInterfaces()) {
@@ -594,6 +576,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
             return serviceProvider instanceof ManagedObjectServiceProvider && ((ManagedObjectServiceProvider) serviceProvider).owner == owner;
         }
 
+        @Override
         public final synchronized void stop() {
             try {
                 if (instance != null) {
@@ -669,10 +652,11 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         }
 
         @Override
-        public void getAll(Class<?> serviceType, List<Service> result) {
+        public Visitor getAll(Class<?> serviceType, ServiceProvider.Visitor visitor) {
             if (serviceType.isAssignableFrom(this.serviceClass)) {
-                result.add(prepare());
+                visitor.visit(prepare());
             }
+            return visitor;
         }
 
         @Override
@@ -717,6 +701,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
 
     private static abstract class FactoryService extends SingletonService {
         private Service[] paramServices;
+        private Service decorates;
 
         protected FactoryService(DefaultServiceRegistry owner, Type serviceType) {
             super(owner, serviceType);
@@ -736,10 +721,9 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
             paramServices = new Service[parameterTypes.length];
             for (int i = 0; i < parameterTypes.length; i++) {
                 Type paramType = parameterTypes[i];
-                if (paramType.equals(ServiceRegistry.class)) {
-                    paramServices[i] = owner.getThisAsService();
-                } else if (paramType.equals(serviceType)) {
-                    Service paramProvider = owner.find(paramType, owner.parentServices);
+                if (paramType.equals(serviceType)) {
+                    // A decorating factory
+                    Service paramProvider = find(paramType, owner.parentServices);
                     if (paramProvider == null) {
                         throw new ServiceCreationException(String.format("Cannot create service of type %s using %s.%s() as required service of type %s is not available in parent registries.",
                             format(serviceType),
@@ -748,10 +732,11 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
                             format(paramType)));
                     }
                     paramServices[i] = paramProvider;
+                    decorates = paramProvider;
                 } else {
                     Service paramProvider;
                     try {
-                        paramProvider = owner.find(paramType, owner.allServices);
+                        paramProvider = find(paramType, owner.allServices);
                     } catch (ServiceLookupException e) {
                         throw new ServiceCreationException(String.format("Cannot create service of type %s using %s.%s() as there is a problem with parameter #%s of type %s.",
                             format(serviceType),
@@ -795,6 +780,24 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
             return params;
         }
 
+        @Override
+        public Visitor getAll(Class<?> serviceType, final Visitor visitor) {
+            super.getAll(serviceType, visitor);
+            if (decorates == null) {
+                return visitor;
+            } else {
+                return new Visitor() {
+                    @Override
+                    public void visit(Service service) {
+                        // Ignore the decorated service
+                        if (service != decorates) {
+                            visitor.visit(service);
+                        }
+                    }
+                };
+            }
+        }
+
         protected abstract Object invokeMethod(Object[] params);
     }
 
@@ -808,10 +811,12 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
             this.method = method;
         }
 
+        @Override
         public String getDisplayName() {
             return "Service " + format(method.getServiceType()) + " at " + method.getOwner().getSimpleName() + "." + method.getName() + "()";
         }
 
+        @Override
         protected Type[] getParameterTypes() {
             return method.getParameterTypes();
         }
@@ -821,6 +826,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
             return method.getMethod();
         }
 
+        @Override
         protected Object invokeMethod(Object[] params) {
             Object result;
             try {
@@ -847,27 +853,13 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         }
     }
 
-    private Service getThisAsService() {
-        return new Service() {
-            public String getDisplayName() {
-                return "ServiceRegistry " + DefaultServiceRegistry.this.getDisplayName();
-            }
-
-            public Object get() {
-                return DefaultServiceRegistry.this;
-            }
-
-            public void requiredBy(ServiceProvider serviceProvider) {
-            }
-        };
-    }
-
     private static class FixedInstanceService<T> extends SingletonService {
         public FixedInstanceService(DefaultServiceRegistry owner, Class<T> serviceType, T serviceInstance) {
             super(owner, serviceType);
             setInstance(serviceInstance);
         }
 
+        @Override
         public String getDisplayName() {
             return "Service " + format(serviceType) + " with implementation " + getInstance();
         }
@@ -884,10 +876,20 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         private ConstructorService(DefaultServiceRegistry owner, Class<?> serviceType) {
             super(owner, serviceType);
             Constructor<?>[] constructors = serviceType.getDeclaredConstructors();
-            if (constructors.length != 1) {
-                throw new ServiceValidationException(String.format("Expected a single constructor for %s.", format(serviceType)));
+            Constructor<?> match = null;
+            for (Constructor<?> constructor : constructors) {
+                if (Modifier.isPrivate(constructor.getModifiers())) {
+                    continue;
+                }
+                if (match != null) {
+                    throw new ServiceValidationException(String.format("Expected a single non-private constructor for %s.", format(serviceType)));
+                }
+                match = constructor;
             }
-            this.constructor = constructors[0];
+            if (match == null) {
+                throw new ServiceValidationException(String.format("Expected a single non-private constructor for %s.", format(serviceType)));
+            }
+            this.constructor = match;
         }
 
         @Override
@@ -911,6 +913,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
             }
         }
 
+        @Override
         public String getDisplayName() {
             return "Service " + format(serviceType);
         }
@@ -946,10 +949,11 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         }
 
         @Override
-        public void getAll(Class<?> serviceType, List<Service> result) {
+        public Visitor getAll(Class<?> serviceType, Visitor visitor) {
             for (ServiceProvider serviceProvider : serviceProviders) {
-                serviceProvider.getAll(serviceType, result);
+                visitor = serviceProvider.getAll(serviceType, visitor);
             }
+            return visitor;
         }
 
         @Override
@@ -963,79 +967,28 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
     }
 
     /**
-     * Allows using a {@link ServiceRegistry} as a provider for another {@link ServiceRegistry},
-     * to create a parent-child relationship. This class is optimized for the case where the
-     * delegate is a {@link DefaultServiceRegistry}, in which case it avoids unnecessary object
-     * creation and exception handling.
+     * Wraps a parent to ignore stop requests.
      */
     private static class ParentServices implements ServiceProvider {
-        private final ServiceRegistry parent;
+        private final ServiceProvider parent;
 
-        private ParentServices(ServiceRegistry parent) {
+        private ParentServices(ServiceProvider parent) {
             this.parent = parent;
         }
 
         @Override
         public Service getFactory(Class<?> serviceType) {
-            if (parent instanceof DefaultServiceRegistry) {
-                return ((DefaultServiceRegistry) parent).getFactoryService(serviceType);
-            }
-            try {
-                Factory<?> factory = parent.getFactory(serviceType);
-                return instanceToService(factory);
-            } catch (UnknownServiceException e) {
-                if (!e.getType().equals(serviceType)) {
-                    throw e;
-                }
-            }
-            return null;
+            return parent.getFactory(serviceType);
         }
 
         @Override
         public Service getService(Type serviceType) {
-            if (parent instanceof DefaultServiceRegistry) {
-                return ((DefaultServiceRegistry) parent).getService(serviceType);
-            }
-            try {
-                Object service = parent.get(serviceType);
-                return instanceToService(service);
-            } catch (UnknownServiceException e) {
-                if (!e.getType().equals(serviceType)) {
-                    throw e;
-                }
-            }
-            return null;
+            return parent.getService(serviceType);
         }
 
         @Override
-        public void getAll(Class<?> serviceType, List<Service> result) {
-            if (parent instanceof DefaultServiceRegistry) {
-                ((DefaultServiceRegistry) parent).collectServices(serviceType, result);
-                return;
-            }
-            List<?> services = parent.getAll(serviceType);
-            for (Object service : services) {
-                result.add(instanceToService(service));
-            }
-        }
-
-        private Service instanceToService(final Object instance) {
-            if (instance == null) {
-                return null;
-            }
-            return new Service() {
-                public String getDisplayName() {
-                    return "ServiceRegistry " + parent;
-                }
-
-                public Object get() {
-                    return instance;
-                }
-
-                public void requiredBy(ServiceProvider serviceProvider) {
-                    // Ignore
-                }
-            };
+        public Visitor getAll(Class<?> serviceType, Visitor visitor) {
+            return parent.getAll(serviceType, visitor);
         }
 
         @Override
@@ -1043,7 +996,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         }
     }
 
-    public Service find(Type serviceType, ServiceProvider serviceProvider) {
+    private static Service find(Type serviceType, ServiceProvider serviceProvider) {
         if (serviceType instanceof ParameterizedType) {
             ParameterizedType parameterizedType = (ParameterizedType) serviceType;
             Type rawType = parameterizedType.getRawType();
@@ -1068,7 +1021,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         throw new ServiceValidationException(String.format("Locating services with type %s is not supported.", format(serviceType)));
     }
 
-    private Service getFactoryService(Type type, ServiceProvider serviceProvider) {
+    private static Service getFactoryService(Type type, ServiceProvider serviceProvider) {
         if (type instanceof Class) {
             return serviceProvider.getFactory((Class) type);
         }
@@ -1088,7 +1041,7 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         throw new ServiceValidationException(String.format("Locating services with type %s is not supported.", format(type)));
     }
 
-    private Service getCollectionService(Type elementType, ServiceProvider serviceProvider) {
+    private static Service getCollectionService(Type elementType, ServiceProvider serviceProvider) {
         if (elementType instanceof Class) {
             Class<?> elementClass = (Class<?>) elementType;
             return getCollectionService(elementClass, serviceProvider);
@@ -1103,10 +1056,10 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
         throw new ServiceValidationException(String.format("Locating services with type %s is not supported.", format(elementType)));
     }
 
-    private Service getCollectionService(Class<?> elementClass, ServiceProvider serviceProvider) {
+    private static Service getCollectionService(Class<?> elementClass, ServiceProvider serviceProvider) {
         assertValidServiceType(elementClass);
         List<Service> providers = new ArrayList<Service>();
-        serviceProvider.getAll(elementClass, providers);
+        serviceProvider.getAll(elementClass, new CollectingVisitor(providers));
         List<Object> services = new ArrayList<Object>(providers.size());
         for (Service service : providers) {
             services.add(service.get());
@@ -1203,7 +1156,12 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
     private static String format(Type type) {
         if (type instanceof Class) {
             Class<?> aClass = (Class) type;
-            return aClass.getSimpleName();
+            Class<?> enclosingClass = aClass.getEnclosingClass();
+            if (enclosingClass != null) {
+                return format(enclosingClass) + "$" + aClass.getSimpleName();
+            } else {
+                return aClass.getSimpleName();
+            }
         } else if (type instanceof ParameterizedType) {
             ParameterizedType parameterizedType = (ParameterizedType) type;
             StringBuilder builder = new StringBuilder();
@@ -1222,4 +1180,45 @@ public class DefaultServiceRegistry implements ServiceRegistry, Closeable {
 
         return type.toString();
     }
+
+    private class ThisAsService implements ServiceProvider, Service {
+        @Override
+        public Service getService(Type serviceType) {
+            if (serviceType.equals(ServiceRegistry.class)) {
+                return this;
+            }
+            return null;
+        }
+
+        @Override
+        public Service getFactory(Class<?> type) {
+            return null;
+        }
+
+        @Override
+        public Visitor getAll(Class<?> serviceType, Visitor visitor) {
+            if (serviceType.equals(ServiceRegistry.class)) {
+                visitor.visit(this);
+            }
+            return visitor;
+        }
+
+        @Override
+        public void stop() {
+        }
+
+        @Override
+        public String getDisplayName() {
+            return "ServiceRegistry " + DefaultServiceRegistry.this.getDisplayName();
+        }
+
+        @Override
+        public Object get() {
+            return DefaultServiceRegistry.this;
+        }
+
+        @Override
+        public void requiredBy(ServiceProvider serviceProvider) {
+        }
+    }
 }
