@@ -18,7 +18,6 @@ package org.gradle.buildinit.tasks;
 
 import org.gradle.api.DefaultTask;
 import org.gradle.api.GradleException;
-import org.gradle.api.Incubating;
 import org.gradle.api.internal.tasks.userinput.UserInputHandler;
 import org.gradle.api.tasks.Input;
 import org.gradle.api.tasks.Internal;
@@ -72,7 +71,6 @@ public class InitBuild extends DefaultTask {
      *
      * @since 4.5
      */
-    @Incubating
     @Optional
     @Input
     public String getDsl() {
@@ -86,7 +84,6 @@ public class InitBuild extends DefaultTask {
      *
      * @since 5.0
      */
-    @Incubating
     @Input
     public String getProjectName() {
         return projectName == null ? getProject().getProjectDir().getName() : projectName;
@@ -99,7 +96,6 @@ public class InitBuild extends DefaultTask {
      *
      * @since 5.0
      */
-    @Incubating
     @Input
     public String getPackageName() {
         return packageName == null ? "" : packageName;
@@ -233,7 +229,6 @@ public class InitBuild extends DefaultTask {
      *
      * @since 4.5
      */
-    @Incubating
     @Option(option = "dsl", description = "Set the build script DSL to be used in generated scripts.")
     public void setDsl(String dsl) {
         this.dsl = dsl;
@@ -244,7 +239,6 @@ public class InitBuild extends DefaultTask {
      *
      * @since 4.5
      */
-    @Incubating
     @OptionValues("dsl")
     @SuppressWarnings("unused")
     public List<String> getAvailableDSLs() {
@@ -273,7 +267,6 @@ public class InitBuild extends DefaultTask {
      *
      * @since 5.0
      */
-    @Incubating
     @Option(option = "project-name", description = "Set the project name.")
     public void setProjectName(String projectName) {
         this.projectName = projectName;
@@ -284,7 +277,6 @@ public class InitBuild extends DefaultTask {
      *
      * @since 5.0
      */
-    @Incubating
     @Option(option = "package", description = "Set the package for source files.")
     public void setPackageName(String packageName) {
         this.packageName = packageName;
