@@ -16,15 +16,13 @@
 
 package org.gradle.composite.internal;
 
-import com.google.common.collect.LinkedHashMultimap;
+import com.google.common.base.MoreObjects;
 import com.google.common.collect.Maps;
-import com.google.common.collect.SetMultimap;
 import org.gradle.api.GradleException;
 import org.gradle.api.artifacts.component.BuildIdentifier;
 import org.gradle.api.internal.BuildDefinition;
 import org.gradle.api.internal.SettingsInternal;
 import org.gradle.api.internal.artifacts.DefaultBuildIdentifier;
-import org.gradle.api.internal.project.ProjectStateRegistry;
 import org.gradle.initialization.GradleLauncherFactory;
 import org.gradle.internal.build.BuildState;
 import org.gradle.internal.build.BuildStateRegistry;
@@ -35,7 +33,6 @@ import org.gradle.internal.build.StandAloneNestedBuild;
 import org.gradle.internal.concurrent.CompositeStoppable;
 import org.gradle.internal.concurrent.Stoppable;
 import org.gradle.internal.event.ListenerManager;
-import org.gradle.internal.logging.text.TreeFormatter;
 import org.gradle.internal.service.scopes.BuildTreeScopeServices;
 import org.gradle.util.Path;
 
@@ -44,11 +41,9 @@ import java.util.ArrayDeque;
 import java.util.Collection;
 import java.util.Deque;
 import java.util.Map;
-import java.util.Set;
 
 public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppable {
     private final IncludedBuildFactory includedBuildFactory;
-    private final ProjectStateRegistry projectRegistry;
     private final IncludedBuildDependencySubstitutionsBuilder dependencySubstitutionsBuilder;
     private final GradleLauncherFactory gradleLauncherFactory;
     private final ListenerManager listenerManager;
@@ -56,13 +51,13 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
 
     // TODO: Locking around this state
     private RootBuildState rootBuild;
-    private final Map<BuildIdentifier, BuildState> builds = Maps.newHashMap();
-    private final Map<File, IncludedBuildState> includedBuilds = Maps.newLinkedHashMap();
+    private final Map<BuildIdentifier, BuildState> buildsByIdentifier = Maps.newHashMap();
+    private final Map<File, IncludedBuildState> includedBuildsByRootDir = Maps.newLinkedHashMap();
+    private final Map<Path, File> includedBuildDirectoriesByPath = Maps.newLinkedHashMap();
     private final Deque<IncludedBuildState> pendingIncludedBuilds = new ArrayDeque<>();
 
-    public DefaultIncludedBuildRegistry(IncludedBuildFactory includedBuildFactory, ProjectStateRegistry projectRegistry, IncludedBuildDependencySubstitutionsBuilder dependencySubstitutionsBuilder, GradleLauncherFactory gradleLauncherFactory, ListenerManager listenerManager, BuildTreeScopeServices rootServices) {
+    public DefaultIncludedBuildRegistry(IncludedBuildFactory includedBuildFactory, IncludedBuildDependencySubstitutionsBuilder dependencySubstitutionsBuilder, GradleLauncherFactory gradleLauncherFactory, ListenerManager listenerManager, BuildTreeScopeServices rootServices) {
         this.includedBuildFactory = includedBuildFactory;
-        this.projectRegistry = projectRegistry;
         this.dependencySubstitutionsBuilder = dependencySubstitutionsBuilder;
         this.gradleLauncherFactory = gradleLauncherFactory;
         this.listenerManager = listenerManager;
@@ -97,29 +92,29 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
     }
 
     private void addBuild(BuildState build) {
-        BuildState before = builds.put(build.getBuildIdentifier(), build);
+        BuildState before = buildsByIdentifier.put(build.getBuildIdentifier(), build);
         if (before != null) {
             throw new IllegalArgumentException("Build is already registered: " + build.getBuildIdentifier());
         }
     }
 
     public boolean hasIncludedBuilds() {
-        return !includedBuilds.isEmpty();
+        return !includedBuildsByRootDir.isEmpty();
     }
 
     @Override
     public Collection<IncludedBuildState> getIncludedBuilds() {
-        return includedBuilds.values();
+        return includedBuildsByRootDir.values();
     }
 
     @Override
     public IncludedBuildState addIncludedBuild(BuildDefinition buildDefinition) {
-         return registerBuild(buildDefinition, false);
+        return registerBuild(buildDefinition, false);
     }
 
     @Override
     public IncludedBuildState getIncludedBuild(final BuildIdentifier buildIdentifier) {
-        BuildState includedBuildState = builds.get(buildIdentifier);
+        BuildState includedBuildState = buildsByIdentifier.get(buildIdentifier);
         if (!(includedBuildState instanceof IncludedBuildState)) {
             throw new IllegalArgumentException("Could not find " + buildIdentifier);
         }
@@ -128,7 +123,7 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
 
     @Override
     public BuildState getBuild(BuildIdentifier buildIdentifier) {
-        BuildState buildState = builds.get(buildIdentifier);
+        BuildState buildState = buildsByIdentifier.get(buildIdentifier);
         if (buildState == null) {
             throw new IllegalArgumentException("Could not find " + buildIdentifier);
         }
@@ -137,36 +132,18 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
 
     @Override
     public void beforeConfigureRootBuild() {
-        registerSubstitutions(includedBuilds.values());
+        registerSubstitutions(includedBuildsByRootDir.values());
     }
 
     @Override
     public void finalizeIncludedBuilds() {
-        SettingsInternal settings = getRootBuild().getLoadedSettings();
-        SetMultimap<String, IncludedBuildState> names = LinkedHashMultimap.create();
+        SettingsInternal rootSettings = getRootBuild().getLoadedSettings();
         while (!pendingIncludedBuilds.isEmpty()) {
             IncludedBuildState build = pendingIncludedBuilds.removeFirst();
-            build.loadSettings();
-            String buildName = build.getName();
-            names.put(buildName, build);
-            if (settings.getRootProject().getName().equals(buildName)) {
-                throw new GradleException("Included build in " + build.getRootDirectory() + " has the same root project name '" + buildName + "' as the main build.");
-            }
-            if (settings.findProject(":" + buildName) != null) {
-                throw new GradleException("Included build in " + build.getRootDirectory() + " has a root project whose name '" + buildName + "' is the same as a project of the main build.");
-            }
-        }
-        for (String buildName : names.keySet()) {
-            Set<IncludedBuildState> buildsWithName = names.get(buildName);
-            if (buildsWithName.size() > 1) {
-                TreeFormatter visitor = new TreeFormatter();
-                visitor.node("Multiple included builds have the same root project name '" + buildName + "'");
-                visitor.startChildren();
-                for (IncludedBuildState build : buildsWithName) {
-                    visitor.node("Included build in " + build.getRootDirectory());
-                }
-                visitor.endChildren();
-                throw new GradleException(visitor.toString());
+            SettingsInternal includedBuildSettings = build.loadSettings();
+            String includedBuildRootProjectName = includedBuildSettings.getRootProject().getName();
+            if (rootSettings.getRootProject().getName().equals(includedBuildRootProjectName)) {
+                throw new GradleException("Included build in " + build.getRootDirectory() + " has the same root project name '" + includedBuildRootProjectName + "' as the main build.");
             }
         }
     }
@@ -180,7 +157,7 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
     @Override
     public synchronized IncludedBuildState addImplicitIncludedBuild(BuildDefinition buildDefinition) {
         // TODO: synchronization with other methods
-        IncludedBuildState includedBuild = includedBuilds.get(buildDefinition.getBuildRootDir());
+        IncludedBuildState includedBuild = includedBuildsByRootDir.get(buildDefinition.getBuildRootDir());
         if (includedBuild == null) {
             includedBuild = registerBuild(buildDefinition, true);
         }
@@ -188,43 +165,57 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
     }
 
     @Override
-    public StandAloneNestedBuild addNestedBuild(BuildDefinition buildDefinition, BuildState owner) {
-        if (buildDefinition.getName() == null) {
-            throw new UnsupportedOperationException("Not yet implemented."); // but should be
+    public StandAloneNestedBuild addBuildSrcNestedBuild(BuildDefinition buildDefinition, BuildState owner) {
+        if (!SettingsInternal.BUILD_SRC.equals(buildDefinition.getName())) {
+            throw new IllegalStateException("Expected buildSrc build, got: " + buildDefinition.getName());
         }
+        Path identityPath = assignPath(owner, buildDefinition.getName(), buildDefinition.getBuildRootDir());
         BuildIdentifier buildIdentifier = idFor(buildDefinition.getName());
-        Path identityPath = pathFor(owner, buildIdentifier.getName());
         DefaultNestedBuild build = new DefaultNestedBuild(buildIdentifier, identityPath, buildDefinition, owner);
         addBuild(build);
         return build;
     }
 
     @Override
-    public NestedRootBuild addNestedBuildTree(BuildDefinition buildDefinition, BuildState owner) {
+    public NestedRootBuild addNestedBuildTree(BuildDefinition buildDefinition, BuildState owner, String buildName) {
         if (buildDefinition.getName() != null || buildDefinition.getBuildRootDir() != null) {
             throw new UnsupportedOperationException("Not yet implemented."); // but should be
         }
-        BuildIdentifier buildIdentifier = idFor(buildDefinition.getStartParameter().getCurrentDir().getName());
-        Path identityPath = pathFor(owner, buildIdentifier.getName());
+        File dir = buildDefinition.getStartParameter().getCurrentDir();
+        String name = MoreObjects.firstNonNull(buildName, dir.getName());
+        validateNameIsNotBuildSrc(name, dir);
+        Path identityPath = assignPath(owner, name, dir);
+        BuildIdentifier buildIdentifier = idFor(name);
         return new RootOfNestedBuildTree(buildDefinition, buildIdentifier, identityPath, owner);
     }
 
+    private void validateNameIsNotBuildSrc(String name, File dir) {
+        if (SettingsInternal.BUILD_SRC.equals(name)) {
+            throw new GradleException("Included build " + dir + " has build name 'buildSrc' which cannot be used as it is a reserved name.");
+        }
+    }
+
     private IncludedBuildState registerBuild(BuildDefinition buildDefinition, boolean isImplicit) {
         // TODO: synchronization
-        if (buildDefinition.getBuildRootDir() == null) {
+        final File buildDir = buildDefinition.getBuildRootDir();
+        if (buildDir == null) {
             throw new IllegalArgumentException("Included build must have a root directory defined");
         }
         if (rootBuild == null) {
             throw new IllegalStateException("No root build attached yet.");
         }
-        IncludedBuildState includedBuild = includedBuilds.get(buildDefinition.getBuildRootDir());
+        IncludedBuildState includedBuild = includedBuildsByRootDir.get(buildDir);
         if (includedBuild == null) {
-            String buildName = buildDefinition.getBuildRootDir().getName();
+            String buildName = buildDefinition.getName();
+            if (buildName == null) {
+                throw new IllegalStateException("build name is required");
+            }
+            validateNameIsNotBuildSrc(buildName, buildDir);
+            Path idPath = assignPath(rootBuild, buildDefinition.getName(), buildDir);
             BuildIdentifier buildIdentifier = idFor(buildName);
-            Path idPath = pathFor(rootBuild, buildIdentifier.getName());
 
             includedBuild = includedBuildFactory.createBuild(buildIdentifier, idPath, buildDefinition, isImplicit, rootBuild);
-            includedBuilds.put(buildDefinition.getBuildRootDir(), includedBuild);
+            includedBuildsByRootDir.put(buildDir, includedBuild);
             pendingIncludedBuilds.add(includedBuild);
             addBuild(includedBuild);
         } else {
@@ -241,18 +232,29 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
 
         // Create a synthetic id for the build, if the id is already used
         // Should instead use a structured id implementation of some kind instead
-        for (int count = 1; builds.containsKey(buildIdentifier); count++) {
+        for (int count = 1; buildsByIdentifier.containsKey(buildIdentifier); count++) {
             buildIdentifier = new DefaultBuildIdentifier(buildName + ":" + count);
         }
         return buildIdentifier;
     }
 
-    private Path pathFor(BuildState owner, String name) {
-        return owner.getIdentityPath().append(Path.path(name));
+    private Path assignPath(BuildState owner, String name, File dir) {
+        Path requestedPath = owner.getIdentityPath().append(Path.path(name));
+        File existingForPath = includedBuildDirectoriesByPath.putIfAbsent(requestedPath, dir);
+        if (existingForPath != null) {
+            throw new GradleException("Included build " + dir + " has build path " + requestedPath + " which is the same as included build " + existingForPath);
+        }
+
+        SettingsInternal rootSettings = getRootBuild().getLoadedSettings();
+        if (rootSettings.findProject(":" + name) != null) {
+            throw new GradleException("Included build in " + dir + " has name '" + name + "' which is the same as a project of the main build.");
+        }
+
+        return requestedPath;
     }
 
     @Override
     public void stop() {
-        CompositeStoppable.stoppable(builds.values()).stop();
+        CompositeStoppable.stoppable(buildsByIdentifier.values()).stop();
     }
 }
