@@ -76,7 +76,7 @@ import java.util.Set;
  * next task by throwing a {@link org.gradle.api.tasks.StopExecutionException}. Using these exceptions allows you to
  * have precondition actions which skip execution of the task, or part of the task, if not true.</p>
  *
- * <a id="dependencies"></a><h3>Task Dependencies and Task Ordering</h3>
+ * <a name="dependencies"></a><h3>Task Dependencies and Task Ordering</h3>
  *
  * <p>A task may have dependencies on other tasks or might be scheduled to always run after another task.
  * Gradle ensures that all task dependencies and ordering rules are honored when executing tasks, so that the task is executed after
@@ -113,13 +113,16 @@ import java.util.Set;
  * <li>A {@code Callable}. The {@code call()} method may return any of the types listed here. Its return value is
  * recursively converted to tasks. A {@code null} return value is treated as an empty collection.</li>
  *
- * <li>Anything else is treated as a failure.</li>
+ * <li>A Groovy {@code Closure} or Kotlin function. The closure or function may return any of the types listed here. Its return value is
+ * recursively converted to tasks. A {@code null} return value is treated as an empty collection.</li>
+ *
+ * <li>Anything else is treated as an error.</li>
  *
  * </ul>
  *
  * <h3>Using a Task in a Build File</h3>
  *
- * <a id="properties"></a> <h4>Dynamic Properties</h4>
+ * <a name="properties"></a> <h4>Dynamic Properties</h4>
  *
  * <p>A {@code Task} has 4 'scopes' for properties. You can access these properties by name from the build file or by
  * calling the {@link #property(String)} method. You can change the value of these properties by calling the {@link #setProperty(String, Object)} method.</p>
@@ -174,7 +177,6 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @since 4.7
      */
-    @Incubating
     String TASK_CONSTRUCTOR_ARGS = "constructorArgs";
 
     /**
@@ -189,6 +191,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      * A {@link org.gradle.api.Namer} namer for tasks that returns {@link #getName()}.
      */
     class Namer implements org.gradle.api.Namer<Task> {
+        @Override
         public String determineName(Task c) {
             return c.getName();
         }
@@ -365,7 +368,6 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @since 4.2
      */
-    @Incubating
     Task doFirst(String actionName, Action<? super Task> action);
 
     /**
@@ -385,7 +387,6 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @since 4.2
      */
-    @Incubating
     Task doLast(String actionName, Action<? super Task> action);
 
     /**
@@ -577,6 +578,7 @@ public interface Task extends Comparable<Task>, ExtensionAware {
      *
      * @since 4.3
      */
+    @Internal
     TaskLocalState getLocalState();
 
     /**
