@@ -28,12 +28,12 @@ import javax.annotation.Nullable;
  * @param <T> the type of elements.
  * @since 4.5
  */
-@Incubating
-public interface HasMultipleValues<T> {
+public interface HasMultipleValues<T> extends HasConfigurableValue {
     /**
      * Sets the value of the property to the elements of the given iterable, and replaces any existing value. This property will query the elements of the iterable each time the value of this property is queried.
      *
-     * <p>This method can also be used to clear the value of the property, by passing {@code null} as the value.
+     * <p>This method can also be used to discard the value of the property, by passing {@code null} as the value.
+     * The convention for this property, if any, will be used to provide the value instead.
      *
      * @param elements The elements, can be null.
      */
@@ -46,12 +46,37 @@ public interface HasMultipleValues<T> {
      */
     void set(Provider<? extends Iterable<? extends T>> provider);
 
+    /**
+     * Sets the value of the property to the elements of the given iterable, and replaces any existing value. This property will query the elements of the iterable each time the value of this property is queried.
+     *
+     * <p>This is the same as {@link #set(Iterable)} but returns this property to allow method chaining.</p>
+     *
+     * @param elements The elements, can be null.
+     * @return this
+     * @since 5.6
+     */
+    @Incubating
+    HasMultipleValues<T> value(@Nullable Iterable<? extends T> elements);
+
+    /**
+     * Sets the property to have the same value of the given provider, and replaces any existing value. This property will track the value of the provider and query its value each time the value of this property is queried. When the provider has no value, this property will also have no value.
+     *
+     * <p>This is the same as {@link #set(Provider)} but returns this property to allow method chaining.</p>
+     *
+     * @param provider Provider of the elements.
+     * @return this
+     * @since 5.6
+     */
+    @Incubating
+    HasMultipleValues<T> value(Provider<? extends Iterable<? extends T>> provider);
+
     /**
      * Sets the value of this property to an empty collection, and replaces any existing value.
      *
      * @return this property.
      * @since 5.0
      */
+    @Incubating
     HasMultipleValues<T> empty();
 
     /**
@@ -99,6 +124,26 @@ public interface HasMultipleValues<T> {
      */
     void addAll(Provider<? extends Iterable<? extends T>> provider);
 
+    /**
+     * Specifies the value to use as the convention for this property. The convention is used when no value has been set for this property.
+     *
+     * @param elements The elements
+     * @return this
+     * @since 5.1
+     */
+    @Incubating
+    HasMultipleValues<T> convention(Iterable<? extends T> elements);
+
+    /**
+     * Specifies the provider of the value to use as the convention for this property. The convention is used when no value has been set for this property.
+     *
+     * @param provider The provider of the elements
+     * @return this
+     * @since 5.1
+     */
+    @Incubating
+    HasMultipleValues<T> convention(Provider<? extends Iterable<? extends T>> provider);
+
     /**
      * Disallows further changes to the value of this property. Calls to methods that change the value of this property, such as {@link #set(Iterable)} or {@link #add(Object)} will fail.
      *
@@ -108,5 +153,6 @@ public interface HasMultipleValues<T> {
      *
      * @since 5.0
      */
+    @Incubating
     void finalizeValue();
 }
