@@ -31,17 +31,19 @@ import org.gradle.api.file.FileCopyDetails;
 import org.gradle.api.file.FileTree;
 import org.gradle.api.file.FileTreeElement;
 import org.gradle.api.file.RelativePath;
-import org.gradle.api.internal.ChainingTransformer;
-import org.gradle.api.internal.ClosureBackedAction;
+import org.gradle.api.internal.file.DefaultCompositeFileTree;
+import org.gradle.api.internal.file.FileCollectionFactory;
 import org.gradle.api.internal.file.FileResolver;
+import org.gradle.api.internal.file.FileTreeInternal;
+import org.gradle.api.internal.file.pattern.PatternMatcher;
 import org.gradle.api.internal.file.pattern.PatternMatcherFactory;
 import org.gradle.api.specs.Spec;
-import org.gradle.api.specs.Specs;
 import org.gradle.api.tasks.util.PatternSet;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.typeconversion.NotationParser;
+import org.gradle.util.ClosureBackedAction;
+import org.gradle.util.CollectionUtils;
 import org.gradle.util.ConfigureUtil;
-import org.gradle.util.DeprecationLogger;
 
 import javax.annotation.Nullable;
 import java.io.File;
@@ -61,24 +63,28 @@ import java.util.regex.Pattern;
 public class DefaultCopySpec implements CopySpecInternal {
     private static final NotationParser<Object, String> PATH_NOTATION_PARSER = PathNotationConverter.parser();
     protected final FileResolver fileResolver;
-    private final Set<Object> sourcePaths = new LinkedHashSet<Object>();
+    protected final FileCollectionFactory fileCollectionFactory;
+    private final Set<Object> sourcePaths = new LinkedHashSet<>();
     private Object destDir;
     private final PatternSet patternSet;
-    private final List<CopySpecInternal> childSpecs = new LinkedList<CopySpecInternal>();
-    private final List<CopySpecInternal> childSpecsInAdditionOrder = new LinkedList<CopySpecInternal>();
+    private final List<CopySpecInternal> childSpecs = new LinkedList<>();
+    private final List<CopySpecInternal> childSpecsInAdditionOrder = new LinkedList<>();
     protected final Instantiator instantiator;
-    private final List<Action<? super FileCopyDetails>> copyActions = new LinkedList<Action<? super FileCopyDetails>>();
+    private final List<Action<? super FileCopyDetails>> copyActions = new LinkedList<>();
     private boolean hasCustomActions;
     private Integer dirMode;
     private Integer fileMode;
     private Boolean caseSensitive;
     private Boolean includeEmptyDirs;
-    private DuplicatesStrategy duplicatesStrategy;
+
+    private DuplicatesStrategy duplicatesStrategy = DuplicatesStrategy.INHERIT;
+
     private String filteringCharset;
     private final List<CopySpecListener> listeners = Lists.newLinkedList();
 
-    public DefaultCopySpec(FileResolver resolver, Instantiator instantiator) {
+    public DefaultCopySpec(FileResolver resolver, FileCollectionFactory fileCollectionFactory, Instantiator instantiator) {
         this.fileResolver = resolver;
+        this.fileCollectionFactory = fileCollectionFactory;
         this.instantiator = instantiator;
         PatternSet patternSet = resolver.getPatternSetFactory().create();
         assert patternSet != null;
@@ -125,24 +131,18 @@ public class DefaultCopySpec implements CopySpecInternal {
     }
 
     @Override
-    public CopySpec from(Object sourcePath, final Closure c) {
-        return from(sourcePath, new ClosureBackedAction<CopySpec>(c));
+    public CopySpec from(Object sourcePath, Closure c) {
+        return from(sourcePath, new ClosureBackedAction<>(c));
     }
 
     @Override
     public CopySpec from(Object sourcePath, Action<? super CopySpec> configureAction) {
-        //noinspection ConstantConditions
-        if (configureAction == null) {
-            DeprecationLogger.nagUserOfDeprecatedBehaviour("Gradle does not allow passing null for the configuration action for CopySpec.from()");
-            from(sourcePath);
-            return this;
-        } else {
-            CopySpecInternal child = addChild();
-            child.from(sourcePath);
-            CopySpecWrapper wrapper = instantiator.newInstance(CopySpecWrapper.class, child);
-            configureAction.execute(wrapper);
-            return wrapper;
-        }
+        Preconditions.checkNotNull(configureAction, "Gradle does not allow passing null for the configuration action for CopySpec.from().");
+        CopySpecInternal child = addChild();
+        child.from(sourcePath);
+        CopySpecWrapper wrapper = instantiator.newInstance(CopySpecWrapper.class, child);
+        configureAction.execute(wrapper);
+        return wrapper;
     }
 
     @Override
@@ -151,14 +151,14 @@ public class DefaultCopySpec implements CopySpecInternal {
     }
 
     protected CopySpecInternal addChildAtPosition(int position) {
-        DefaultCopySpec child = instantiator.newInstance(SingleParentCopySpec.class, fileResolver, instantiator, buildRootResolver());
+        DefaultCopySpec child = instantiator.newInstance(SingleParentCopySpec.class, fileResolver, fileCollectionFactory, instantiator, buildRootResolver());
         addChildSpec(position, child);
         return child;
     }
 
     @Override
     public CopySpecInternal addChild() {
-        DefaultCopySpec child = new SingleParentCopySpec(fileResolver, instantiator, buildRootResolver());
+        DefaultCopySpec child = new SingleParentCopySpec(fileResolver, fileCollectionFactory, instantiator, buildRootResolver());
         addChildSpec(child);
         return child;
     }
@@ -181,21 +181,13 @@ public class DefaultCopySpec implements CopySpecInternal {
         childSpecsInAdditionOrder.add(childSpec);
 
         // In case more descendants are added to downward hierarchy, make sure they'll notify us
-        childSpec.addChildSpecListener(new CopySpecListener() {
-            @Override
-            public void childSpecAdded(CopySpecAddress path, CopySpecInternal spec) {
-                CopySpecAddress childPath = new DefaultCopySpecAddress(null, DefaultCopySpec.this, additionIndex).append(path);
-                fireChildSpecListeners(childPath, spec);
-            }
+        childSpec.addChildSpecListener((path, spec) -> {
+            CopySpecAddress childPath = new DefaultCopySpecAddress(null, DefaultCopySpec.this, additionIndex).append(path);
+            fireChildSpecListeners(childPath, spec);
         });
 
         // Notify upwards of currently existing descendant spec hierarchy
-        childSpec.visit(new DefaultCopySpecAddress(null, this, additionIndex), new CopySpecVisitor() {
-            @Override
-            public void visit(final CopySpecAddress parentPath, CopySpecInternal spec) {
-                fireChildSpecListeners(parentPath, spec);
-            }
-        });
+        childSpec.visit(new DefaultCopySpecAddress(null, this, additionIndex), this::fireChildSpecListeners);
     }
 
     private void fireChildSpecListeners(CopySpecAddress path, CopySpecInternal spec) {
@@ -234,23 +226,17 @@ public class DefaultCopySpec implements CopySpecInternal {
 
     @Override
     public CopySpec into(Object destPath, Closure configureClosure) {
-        return into(destPath, new ClosureBackedAction<CopySpec>(configureClosure));
+        return into(destPath, new ClosureBackedAction<>(configureClosure));
     }
 
     @Override
     public CopySpec into(Object destPath, Action<? super CopySpec> copySpec) {
-        //noinspection ConstantConditions
-        if (copySpec == null) {
-            DeprecationLogger.nagUserOfDeprecatedBehaviour("Gradle does not allow passing null for the configuration action for CopySpec.into()");
-            into(destPath);
-            return this;
-        } else {
-            CopySpecInternal child = addChild();
-            child.into(destPath);
-            CopySpecWrapper wrapper = instantiator.newInstance(CopySpecWrapper.class, child);
-            copySpec.execute(wrapper);
-            return wrapper;
-        }
+        Preconditions.checkNotNull(copySpec, "Gradle does not allow passing null for the configuration action for CopySpec.into().");
+        CopySpecInternal child = addChild();
+        child.into(destPath);
+        CopySpecWrapper wrapper = instantiator.newInstance(CopySpecWrapper.class, child);
+        copySpec.execute(wrapper);
+        return wrapper;
     }
 
     @Override
@@ -279,13 +265,13 @@ public class DefaultCopySpec implements CopySpecInternal {
     }
 
     @Override
-    public void setDuplicatesStrategy(@Nullable DuplicatesStrategy strategy) {
+    public void setDuplicatesStrategy(DuplicatesStrategy strategy) {
         this.duplicatesStrategy = strategy;
     }
 
     @Override
     public CopySpec filesMatching(String pattern, Action<? super FileCopyDetails> action) {
-        Spec<RelativePath> matcher = PatternMatcherFactory.getPatternMatcher(true, isCaseSensitive(), pattern);
+        PatternMatcher matcher = PatternMatcherFactory.getPatternMatcher(true, isCaseSensitive(), pattern);
         return eachFile(new MatchingCopyAction(matcher, action));
     }
 
@@ -294,17 +280,14 @@ public class DefaultCopySpec implements CopySpecInternal {
         if (!patterns.iterator().hasNext()) {
             throw new InvalidUserDataException("must provide at least one pattern to match");
         }
-        List<Spec<RelativePath>> matchers = new ArrayList<Spec<RelativePath>>();
-        for (String pattern : patterns) {
-            matchers.add(PatternMatcherFactory.getPatternMatcher(true, isCaseSensitive(), pattern));
-        }
-        return eachFile(new MatchingCopyAction(Specs.union(matchers), action));
+        PatternMatcher matcher = PatternMatcherFactory.getPatternsMatcher(true, isCaseSensitive(), patterns);
+        return eachFile(new MatchingCopyAction(matcher, action));
     }
 
     @Override
     public CopySpec filesNotMatching(String pattern, Action<? super FileCopyDetails> action) {
-        Spec<RelativePath> matcher = PatternMatcherFactory.getPatternMatcher(true, isCaseSensitive(), pattern);
-        return eachFile(new MatchingCopyAction(Specs.negate(matcher), action));
+        PatternMatcher matcher = PatternMatcherFactory.getPatternMatcher(true, isCaseSensitive(), pattern);
+        return eachFile(new MatchingCopyAction(matcher.negate(), action));
     }
 
     @Override
@@ -312,11 +295,8 @@ public class DefaultCopySpec implements CopySpecInternal {
         if (!patterns.iterator().hasNext()) {
             throw new InvalidUserDataException("must provide at least one pattern to not match");
         }
-        List<Spec<RelativePath>> matchers = new ArrayList<Spec<RelativePath>>();
-        for (String pattern : patterns) {
-            matchers.add(PatternMatcherFactory.getPatternMatcher(true, isCaseSensitive(), pattern));
-        }
-        return eachFile(new MatchingCopyAction(Specs.negate(Specs.union(matchers)), action));
+        PatternMatcher matcher = PatternMatcherFactory.getPatternsMatcher(true, isCaseSensitive(), patterns);
+        return eachFile(new MatchingCopyAction(matcher.negate(), action));
     }
 
     @Override
@@ -403,11 +383,7 @@ public class DefaultCopySpec implements CopySpecInternal {
 
     @Override
     public CopySpec filter(final Class<? extends FilterReader> filterType) {
-        appendCopyAction(new Action<FileCopyDetails>() {
-            public void execute(FileCopyDetails fileCopyDetails) {
-                fileCopyDetails.filter(filterType);
-            }
-        });
+        appendCopyAction(fileCopyDetails -> fileCopyDetails.filter(filterType));
         return this;
     }
 
@@ -418,31 +394,19 @@ public class DefaultCopySpec implements CopySpecInternal {
 
     @Override
     public CopySpec filter(final Transformer<String, String> transformer) {
-        appendCopyAction(new Action<FileCopyDetails>() {
-            public void execute(FileCopyDetails fileCopyDetails) {
-                fileCopyDetails.filter(transformer);
-            }
-        });
+        appendCopyAction(fileCopyDetails -> fileCopyDetails.filter(transformer));
         return this;
     }
 
     @Override
     public CopySpec filter(final Map<String, ?> properties, final Class<? extends FilterReader> filterType) {
-        appendCopyAction(new Action<FileCopyDetails>() {
-            public void execute(FileCopyDetails fileCopyDetails) {
-                fileCopyDetails.filter(properties, filterType);
-            }
-        });
+        appendCopyAction(fileCopyDetails -> fileCopyDetails.filter(properties, filterType));
         return this;
     }
 
     @Override
     public CopySpec expand(final Map<String, ?> properties) {
-        appendCopyAction(new Action<FileCopyDetails>() {
-            public void execute(FileCopyDetails fileCopyDetails) {
-                fileCopyDetails.expand(properties);
-            }
-        });
+        appendCopyAction(fileCopyDetails -> fileCopyDetails.expand(properties));
         return this;
     }
 
@@ -453,7 +417,7 @@ public class DefaultCopySpec implements CopySpecInternal {
 
     @Override
     public CopySpec rename(Transformer<String, String> renamer) {
-        ChainingTransformer<String> transformer = new ChainingTransformer<String>(String.class);
+        ChainingTransformer<String> transformer = new ChainingTransformer<>(String.class);
         transformer.add(renamer);
         appendCopyAction(new RenamingCopyAction(transformer));
         return this;
@@ -520,10 +484,14 @@ public class DefaultCopySpec implements CopySpecInternal {
 
     @Override
     public CopySpecResolver buildRootResolver() {
-        //noinspection ConstantConditions
         return this.new DefaultCopySpecResolver(null);
     }
 
+    // TODO:instant-execution - remove this
+    public Set<File> resolveSourceFiles() {
+        return fileCollectionFactory.resolving(sourcePaths).getFiles();
+    }
+
     @Override
     public String getFilteringCharset() {
         return buildRootResolver().getFilteringCharset();
@@ -542,9 +510,7 @@ public class DefaultCopySpec implements CopySpecInternal {
 
         private CopySpecResolver parentResolver;
 
-        // Not marked as @Nullable because of Groovy compiler bug: https://issues.apache.org/jira/browse/GROOVY-8505
-        @SuppressWarnings("NullableProblems")
-        private DefaultCopySpecResolver(/* @Nullable */ CopySpecResolver parent) {
+        private DefaultCopySpecResolver(@Nullable CopySpecResolver parent) {
             this.parentResolver = parent;
         }
 
@@ -572,19 +538,15 @@ public class DefaultCopySpec implements CopySpecInternal {
 
         @Override
         public FileTree getSource() {
-            return fileResolver.resolveFilesAsTree(sourcePaths).matching(this.getPatternSet());
+            return fileCollectionFactory.resolving(sourcePaths).getAsFileTree().matching(this.getPatternSet());
         }
 
         @Override
         public FileTree getAllSource() {
             final ImmutableList.Builder<FileTree> builder = ImmutableList.builder();
-            walk(new Action<CopySpecResolver>() {
-                public void execute(CopySpecResolver copySpecResolver) {
-                    builder.add(copySpecResolver.getSource());
-                }
-            });
+            walk(copySpecResolver -> builder.add(copySpecResolver.getSource()));
 
-            return fileResolver.compositeFileTree(builder.build());
+            return new DefaultCompositeFileTree(CollectionUtils.checkedCast(FileTreeInternal.class, builder.build()));
         }
 
         @Override
@@ -592,7 +554,7 @@ public class DefaultCopySpec implements CopySpecInternal {
             if (parentResolver == null) {
                 return copyActions;
             }
-            List<Action<? super FileCopyDetails>> allActions = new ArrayList<Action<? super FileCopyDetails>>();
+            List<Action<? super FileCopyDetails>> allActions = new ArrayList<>();
             allActions.addAll(parentResolver.getAllCopyActions());
             allActions.addAll(copyActions);
             return allActions;
@@ -600,7 +562,7 @@ public class DefaultCopySpec implements CopySpecInternal {
 
         @Override
         public List<String> getAllIncludes() {
-            List<String> result = new ArrayList<String>();
+            List<String> result = new ArrayList<>();
             if (parentResolver != null) {
                 result.addAll(parentResolver.getAllIncludes());
             }
@@ -610,7 +572,7 @@ public class DefaultCopySpec implements CopySpecInternal {
 
         @Override
         public List<String> getAllExcludes() {
-            List<String> result = new ArrayList<String>();
+            List<String> result = new ArrayList<>();
             if (parentResolver != null) {
                 result.addAll(parentResolver.getAllExcludes());
             }
@@ -621,7 +583,7 @@ public class DefaultCopySpec implements CopySpecInternal {
 
         @Override
         public List<Spec<FileTreeElement>> getAllExcludeSpecs() {
-            List<Spec<FileTreeElement>> result = new ArrayList<Spec<FileTreeElement>>();
+            List<Spec<FileTreeElement>> result = new ArrayList<>();
             if (parentResolver != null) {
                 result.addAll(parentResolver.getAllExcludeSpecs());
             }
@@ -631,7 +593,7 @@ public class DefaultCopySpec implements CopySpecInternal {
 
         @Override
         public DuplicatesStrategy getDuplicatesStrategy() {
-            if (duplicatesStrategy != null) {
+            if (duplicatesStrategy != DuplicatesStrategy.INHERIT) {
                 return duplicatesStrategy;
             }
             if (parentResolver != null) {
@@ -640,6 +602,17 @@ public class DefaultCopySpec implements CopySpecInternal {
             return DuplicatesStrategy.INCLUDE;
         }
 
+        @Override
+        public boolean isDefaultDuplicateStrategy() {
+            if (duplicatesStrategy != DuplicatesStrategy.INHERIT) {
+                return false;
+            }
+            if (parentResolver != null) {
+                return parentResolver.isDefaultDuplicateStrategy();
+            }
+            return true;
+        }
+
         @Override
         public boolean isCaseSensitive() {
             if (caseSensitive != null) {
@@ -673,6 +646,7 @@ public class DefaultCopySpec implements CopySpecInternal {
             return null;
         }
 
+        @Override
         public boolean getIncludeEmptyDirs() {
             if (includeEmptyDirs != null) {
                 return includeEmptyDirs;
@@ -683,8 +657,9 @@ public class DefaultCopySpec implements CopySpecInternal {
             return true;
         }
 
+        @Override
         public List<Spec<FileTreeElement>> getAllIncludeSpecs() {
-            List<Spec<FileTreeElement>> result = new ArrayList<Spec<FileTreeElement>>();
+            List<Spec<FileTreeElement>> result = new ArrayList<>();
             if (parentResolver != null) {
                 result.addAll(parentResolver.getAllIncludeSpecs());
             }
@@ -703,6 +678,7 @@ public class DefaultCopySpec implements CopySpecInternal {
             return patterns;
         }
 
+        @Override
         public void walk(Action<? super CopySpecResolver> action) {
             action.execute(this);
             for (CopySpecInternal child : getChildren()) {
@@ -710,6 +686,7 @@ public class DefaultCopySpec implements CopySpecInternal {
             }
         }
 
+        @Override
         public String getFilteringCharset() {
             if (filteringCharset != null) {
                 return filteringCharset;
@@ -719,8 +696,6 @@ public class DefaultCopySpec implements CopySpecInternal {
             }
             return Charset.defaultCharset().name();
         }
-
-
     }
 
     private class DefaultCopySpecAddress implements CopySpecAddress {
