@@ -23,19 +23,20 @@ import org.gradle.api.file.RegularFile;
 import org.gradle.api.file.RegularFileProperty;
 import org.gradle.api.internal.file.copy.CopyActionExecuter;
 import org.gradle.api.model.ObjectFactory;
+import org.gradle.api.model.ReplacedBy;
 import org.gradle.api.provider.Property;
 import org.gradle.api.provider.Provider;
 import org.gradle.api.tasks.AbstractCopyTask;
 import org.gradle.api.tasks.Input;
 import org.gradle.api.tasks.Internal;
 import org.gradle.api.tasks.OutputFile;
-import org.gradle.internal.nativeplatform.filesystem.FileSystem;
+import org.gradle.internal.nativeintegration.filesystem.FileSystem;
 import org.gradle.internal.reflect.Instantiator;
+import org.gradle.util.DeprecationLogger;
 import org.gradle.util.GUtil;
 
 import javax.annotation.Nullable;
 import java.io.File;
-import java.util.concurrent.Callable;
 
 /**
  * {@code AbstractArchiveTask} is the base class for all archive tasks.
@@ -63,29 +64,26 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
         archiveAppendix = objectFactory.property(String.class);
         archiveVersion = objectFactory.property(String.class);
         archiveExtension = objectFactory.property(String.class);
-        archiveClassifier = objectFactory.property(String.class).value("");
+        archiveClassifier = objectFactory.property(String.class).convention("");
 
         archiveName = objectFactory.property(String.class);
-        archiveName.set(getProject().provider(new Callable<String>() {
-            @Override
-            public String call() throws Exception {
-                // [baseName]-[appendix]-[version]-[classifier].[extension]
-                String name = GUtil.elvis(archiveBaseName.getOrNull(), "");
-                name += maybe(name, archiveAppendix.getOrNull());
-                name += maybe(name, archiveVersion.getOrNull());
-                name += maybe(name, archiveClassifier.getOrNull());
+        archiveName.convention(getProject().provider(() -> {
+            // [baseName]-[appendix]-[version]-[classifier].[extension]
+            String name = GUtil.elvis(archiveBaseName.getOrNull(), "");
+            name += maybe(name, archiveAppendix.getOrNull());
+            name += maybe(name, archiveVersion.getOrNull());
+            name += maybe(name, archiveClassifier.getOrNull());
 
-                String extension = archiveExtension.getOrNull();
-                name += GUtil.isTrue(extension) ? "." + extension : "";
-                return name;
-            }
+            String extension = archiveExtension.getOrNull();
+            name += GUtil.isTrue(extension) ? "." + extension : "";
+            return name;
         }));
 
         archiveFile = objectFactory.fileProperty();
-        archiveFile.set(archiveDestinationDirectory.file(archiveName));
+        archiveFile.convention(archiveDestinationDirectory.file(archiveName));
 
-        archivePreserveFileTimestamps = objectFactory.property(Boolean.class).value(true);
-        archiveReproducibleFileOrder = objectFactory.property(Boolean.class).value(false);
+        archivePreserveFileTimestamps = objectFactory.property(Boolean.class).convention(true);
+        archiveReproducibleFileOrder = objectFactory.property(Boolean.class).convention(false);
     }
 
     private static String maybe(@Nullable String prefix, @Nullable String value) {
@@ -107,8 +105,10 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      * @deprecated Use {@link #getArchiveFileName()}
      */
     @Deprecated
-    @Internal("Represented as part of archiveFile")
+    @ReplacedBy("archiveFileName")
     public String getArchiveName() {
+        // This is used by the Kotlin plugin, we should upstream a fix to avoid this API first.
+        // DeprecationLogger.nagUserOfReplacedProperty("archiveName", "archiveFileName");
         return archiveName.get();
     }
 
@@ -120,6 +120,8 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      */
     @Deprecated
     public void setArchiveName(String name) {
+        DeprecationLogger.nagUserOfReplacedProperty("archiveName", "archiveFileName");
+        archiveName.convention(name);
         archiveName.set(name);
     }
 
@@ -142,8 +144,10 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      * @deprecated Use {@link #getArchiveFile()}
      */
     @Deprecated
-    @Internal("Represented as a part of the archiveFile")
+    @ReplacedBy("archiveFile")
     public File getArchivePath() {
+        // This is used by the Kotlin plugin, we should upstream a fix to avoid this API first.
+        // DeprecationLogger.nagUserOfReplacedProperty("archivePath", "archiveFile");
         return getArchiveFile().get().getAsFile();
     }
 
@@ -175,9 +179,10 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      * @return the directory
      * @deprecated Use {@link #getDestinationDirectory()}
      */
-    @Internal("Represented as part of archiveFile")
     @Deprecated
+    @ReplacedBy("destinationDirectory")
     public File getDestinationDir() {
+        DeprecationLogger.nagUserOfReplacedProperty("destinationDir", "destinationDirectory");
         return archiveDestinationDirectory.getAsFile().get();
     }
 
@@ -188,7 +193,8 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      */
     @Deprecated
     public void setDestinationDir(File destinationDir) {
-        archiveDestinationDirectory.set(destinationDir);
+        DeprecationLogger.nagUserOfReplacedProperty("destinationDir", "destinationDirectory");
+        archiveDestinationDirectory.set(getProject().file(destinationDir));
     }
 
     /**
@@ -208,9 +214,10 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      * @deprecated Use {@link #getArchiveBaseName()}
      */
     @Nullable
-    @Internal("Represented as part of archiveFile")
     @Deprecated
+    @ReplacedBy("archiveBaseName")
     public String getBaseName() {
+        DeprecationLogger.nagUserOfReplacedProperty("baseName", "archiveBaseName");
         return archiveBaseName.getOrNull();
     }
 
@@ -221,7 +228,9 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      */
     @Deprecated
     public void setBaseName(@Nullable String baseName) {
-        this.archiveBaseName.set(baseName);
+        DeprecationLogger.nagUserOfReplacedProperty("baseName", "archiveBaseName");
+        archiveBaseName.convention(baseName);
+        archiveBaseName.set(baseName);
     }
 
     /**
@@ -242,9 +251,10 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      * @deprecated Use {@link #getArchiveAppendix()}
      */
     @Nullable
-    @Internal("Represented as part of archiveFile")
     @Deprecated
+    @ReplacedBy("archiveAppendix")
     public String getAppendix() {
+        DeprecationLogger.nagUserOfReplacedProperty("appendix", "archiveAppendix");
         return archiveAppendix.getOrNull();
     }
 
@@ -255,7 +265,10 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      */
     @Deprecated
     public void setAppendix(@Nullable String appendix) {
-        this.archiveAppendix.set(appendix);
+        // This is used by the Kotlin plugin, we should upstream a fix to avoid this API first.
+        // DeprecationLogger.nagUserOfReplacedProperty("appendix", "archiveAppendix");
+        archiveAppendix.convention(appendix);
+        archiveAppendix.set(appendix);
     }
 
     /**
@@ -276,9 +289,10 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      * @deprecated Use {@link #getArchiveVersion()}
      */
     @Nullable
-    @Internal("Represented as part of archiveFile")
     @Deprecated
+    @ReplacedBy("archiveVersion")
     public String getVersion() {
+        DeprecationLogger.nagUserOfReplacedProperty("version", "archiveVersion");
         return archiveVersion.getOrNull();
     }
 
@@ -289,7 +303,9 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      */
     @Deprecated
     public void setVersion(@Nullable String version) {
-        this.archiveVersion.set(version);
+        DeprecationLogger.nagUserOfReplacedProperty("version", "archiveVersion");
+        archiveVersion.convention(version);
+        archiveVersion.set(version);
     }
 
     /**
@@ -300,7 +316,7 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      */
     @Internal("Represented as part of archiveFile")
     public Property<String> getArchiveVersion() {
-        return this.archiveVersion;
+        return archiveVersion;
     }
 
     /**
@@ -308,9 +324,10 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      * @deprecated Use {@link #getArchiveExtension()}
      */
     @Nullable
-    @Internal("Represented as part of archiveFile")
     @Deprecated
+    @ReplacedBy("archiveExtension")
     public String getExtension() {
+        DeprecationLogger.nagUserOfReplacedProperty("extension", "archiveExtension");
         return archiveExtension.getOrNull();
     }
 
@@ -321,7 +338,9 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      */
     @Deprecated
     public void setExtension(@Nullable String extension) {
-        this.archiveExtension.set(extension);
+        DeprecationLogger.nagUserOfReplacedProperty("extension", "archiveExtension");
+        archiveExtension.convention(extension);
+        archiveExtension.set(extension);
     }
 
     /**
@@ -340,9 +359,10 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      * @deprecated Use {@link #getArchiveClassifier()}
      */
     @Nullable
-    @Internal("Represented as part of archiveFile")
     @Deprecated
+    @ReplacedBy("archiveClassifier")
     public String getClassifier() {
+        DeprecationLogger.nagUserOfReplacedProperty("classifier", "archiveClassifier");
         return archiveClassifier.getOrNull();
     }
 
@@ -353,7 +373,10 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      */
     @Deprecated
     public void setClassifier(@Nullable String classifier) {
-        this.archiveClassifier.set(classifier);
+        // This is used by the Kotlin plugin, we should upstream a fix to avoid this API first.
+        // DeprecationLogger.nagUserOfReplacedProperty("classifier", "archiveClassifier");
+        archiveClassifier.convention(classifier);
+        archiveClassifier.set(classifier);
     }
 
     /**
@@ -437,7 +460,7 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      * @param preserveFileTimestamps <tt>true</tt> if file timestamps should be preserved for archive entries
      */
     public void setPreserveFileTimestamps(boolean preserveFileTimestamps) {
-        this.archivePreserveFileTimestamps.set(preserveFileTimestamps);
+        archivePreserveFileTimestamps.set(preserveFileTimestamps);
     }
 
     /**
@@ -467,7 +490,7 @@ public abstract class AbstractArchiveTask extends AbstractCopyTask {
      * @param reproducibleFileOrder <tt>true</tt> if the files should read from disk in a reproducible order.
      */
     public void setReproducibleFileOrder(boolean reproducibleFileOrder) {
-        this.archiveReproducibleFileOrder.set(reproducibleFileOrder);
+        archiveReproducibleFileOrder.set(reproducibleFileOrder);
     }
 
     @Override
