@@ -22,10 +22,12 @@ import org.gradle.BuildListener;
 import org.gradle.BuildResult;
 import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.internal.SettingsInternal;
+import org.gradle.api.internal.initialization.ClassLoaderScope;
 import org.gradle.composite.internal.IncludedBuildControllers;
-import org.gradle.configuration.BuildConfigurer;
-import org.gradle.execution.BuildExecuter;
+import org.gradle.configuration.ProjectsPreparer;
+import org.gradle.execution.BuildWorkExecutor;
 import org.gradle.execution.MultipleBuildFailures;
+import org.gradle.initialization.buildsrc.BuildSourceBuilder;
 import org.gradle.initialization.exception.ExceptionAnalyser;
 import org.gradle.internal.concurrent.CompositeStoppable;
 import org.gradle.internal.service.scopes.BuildScopeServices;
@@ -49,29 +51,30 @@ public class DefaultGradleLauncher implements GradleLauncher {
         }
     }
 
-    private final BuildConfigurer buildConfigurer;
+    private final ProjectsPreparer projectsPreparer;
     private final ExceptionAnalyser exceptionAnalyser;
     private final BuildListener buildListener;
     private final BuildCompletionListener buildCompletionListener;
-    private final BuildExecuter buildExecuter;
+    private final BuildWorkExecutor buildExecuter;
     private final BuildScopeServices buildServices;
     private final List<?> servicesToStop;
     private final IncludedBuildControllers includedBuildControllers;
     private final GradleInternal gradle;
-    private Stage stage;
-
     private final InstantExecution instantExecution;
     private final SettingsPreparer settingsPreparer;
     private final TaskExecutionPreparer taskExecutionPreparer;
+    private final BuildSourceBuilder buildSourceBuilder;
 
-    public DefaultGradleLauncher(GradleInternal gradle, BuildConfigurer buildConfigurer, ExceptionAnalyser exceptionAnalyser,
+    private Stage stage;
+
+    public DefaultGradleLauncher(GradleInternal gradle, ProjectsPreparer projectsPreparer, ExceptionAnalyser exceptionAnalyser,
                                  BuildListener buildListener, BuildCompletionListener buildCompletionListener,
-                                 BuildExecuter buildExecuter, BuildScopeServices buildServices,
+                                 BuildWorkExecutor buildExecuter, BuildScopeServices buildServices,
                                  List<?> servicesToStop, IncludedBuildControllers includedBuildControllers,
                                  SettingsPreparer settingsPreparer, TaskExecutionPreparer taskExecutionPreparer,
-                                 InstantExecution instantExecution) {
+                                 InstantExecution instantExecution, BuildSourceBuilder buildSourceBuilder) {
         this.gradle = gradle;
-        this.buildConfigurer = buildConfigurer;
+        this.projectsPreparer = projectsPreparer;
         this.exceptionAnalyser = exceptionAnalyser;
         this.buildListener = buildListener;
         this.buildExecuter = buildExecuter;
@@ -82,6 +85,7 @@ public class DefaultGradleLauncher implements GradleLauncher {
         this.instantExecution = instantExecution;
         this.settingsPreparer = settingsPreparer;
         this.taskExecutionPreparer = taskExecutionPreparer;
+        this.buildSourceBuilder = buildSourceBuilder;
     }
 
     @Override
@@ -101,6 +105,7 @@ public class DefaultGradleLauncher implements GradleLauncher {
         return gradle;
     }
 
+    @Override
     public GradleInternal executeTasks() {
         doBuildStages(Stage.RunTasks);
         return gradle;
@@ -134,7 +139,7 @@ public class DefaultGradleLauncher implements GradleLauncher {
         if (upTo == Stage.LoadSettings) {
             return;
         }
-        configureBuild();
+        prepareProjects();
         if (upTo == Stage.Configure) {
             return;
         }
@@ -142,15 +147,15 @@ public class DefaultGradleLauncher implements GradleLauncher {
         if (upTo == Stage.TaskGraph) {
             return;
         }
-        instantExecution.saveTaskGraph();
-        runTasks();
+        instantExecution.saveScheduledWork();
+        runWork();
     }
 
     private void doInstantExecution() {
         buildListener.buildStarted(gradle);
-        instantExecution.loadTaskGraph();
+        instantExecution.loadScheduledWork();
         stage = Stage.TaskGraph;
-        runTasks();
+        runWork();
     }
 
     private void finishBuild(String action, @Nullable Throwable stageFailure) {
@@ -192,10 +197,11 @@ public class DefaultGradleLauncher implements GradleLauncher {
         }
     }
 
-    private void configureBuild() {
+    private void prepareProjects() {
         if (stage == Stage.LoadSettings) {
-            buildConfigurer.configure(gradle);
-
+            ClassLoaderScope baseProjectClassLoaderScope = buildSourceBuilder.buildAndCreateClassLoader(gradle);
+            gradle.setBaseProjectClassLoaderScope(baseProjectClassLoaderScope);
+            projectsPreparer.prepareProjects(gradle);
             stage = Stage.Configure;
         }
     }
@@ -226,7 +232,7 @@ public class DefaultGradleLauncher implements GradleLauncher {
         doBuildStages(Stage.TaskGraph);
     }
 
-    private void runTasks() {
+    private void runWork() {
         if (stage != Stage.TaskGraph) {
             throw new IllegalStateException("Cannot execute tasks: current stage = " + stage);
         }
@@ -241,8 +247,8 @@ public class DefaultGradleLauncher implements GradleLauncher {
     }
 
     /**
-     * <p>Adds a listener to this build instance. The listener is notified of events which occur during the execution of the build. See {@link org.gradle.api.invocation.Gradle#addListener(Object)} for
-     * supported listener types.</p>
+     * <p>Adds a listener to this build instance. The listener is notified of events which occur during the execution of the build.
+     * See {@link org.gradle.api.invocation.Gradle#addListener(Object)} for supported listener types.</p>
      *
      * @param listener The listener to add. Has no effect if the listener has already been added.
      */
@@ -251,6 +257,7 @@ public class DefaultGradleLauncher implements GradleLauncher {
         gradle.addListener(listener);
     }
 
+    @Override
     public void stop() {
         try {
             CompositeStoppable.stoppable(buildServices).add(servicesToStop).stop();
