@@ -78,7 +78,7 @@ public class CrossBuildSessionScopeServices implements Closeable {
         this.buildOperationListenerManager = parent.get(BuildOperationListenerManager.class);
 
         ListenerManager generalListenerManager = parent.get(ListenerManager.class);
-        this.buildOperationTrace = new BuildOperationTrace(startParameter, buildOperationListenerManager, generalListenerManager);
+        this.buildOperationTrace = new BuildOperationTrace(startParameter, buildOperationListenerManager);
         this.buildOperationNotificationBridge = new BuildOperationNotificationBridge(buildOperationListenerManager, generalListenerManager);
         this.loggingBuildOperationProgressBroadcaster = new LoggingBuildOperationProgressBroadcaster(parent.get(OutputEventListenerManager.class), buildOperationListenerManager.getBroadcaster());
     }
@@ -95,6 +95,10 @@ public class CrossBuildSessionScopeServices implements Closeable {
         return buildOperationListenerManager;
     }
 
+    LoggingBuildOperationProgressBroadcaster createLoggingBuildOperationProgressBroadcaster() {
+        return loggingBuildOperationProgressBroadcaster;
+    }
+
     BuildOperationExecutor createBuildOperationExecutor() {
         // Wrap to prevent exposing Stoppable, as we don't want to stop at this scope
         return new DelegatingBuildOperationExecutor(services.get(BuildOperationExecutor.class));
