@@ -17,9 +17,19 @@ package org.gradle.api.internal.artifacts;
 
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelector;
 
+import javax.annotation.Nullable;
+
 public interface ResolvedVersionConstraint {
+    @Nullable
     VersionSelector getPreferredSelector();
+
+    @Nullable
     VersionSelector getRequiredSelector();
+
+    @Nullable
     VersionSelector getRejectedSelector();
+
     boolean isRejectAll();
+    boolean isDynamic();
+    boolean isStrict();
 }
