@@ -16,8 +16,6 @@
 
 package org.gradle.api.internal.artifacts.ivyservice.ivyresolve;
 
-import org.apache.http.HttpStatus;
-import org.apache.http.conn.HttpHostConnectException;
 import org.gradle.api.Transformer;
 import org.gradle.api.artifacts.ComponentMetadataSupplierDetails;
 import org.gradle.api.artifacts.component.ComponentArtifactIdentifier;
@@ -25,6 +23,7 @@ import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 import org.gradle.api.artifacts.component.ModuleComponentSelector;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvableArtifact;
 import org.gradle.api.internal.artifacts.repositories.resolver.MetadataFetchingCost;
+import org.gradle.api.internal.artifacts.repositories.transport.NetworkingIssueVerifier;
 import org.gradle.api.internal.component.ArtifactType;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
@@ -35,8 +34,8 @@ import org.gradle.internal.component.external.model.ModuleDependencyMetadata;
 import org.gradle.internal.component.model.ComponentArtifactMetadata;
 import org.gradle.internal.component.model.ComponentOverrideMetadata;
 import org.gradle.internal.component.model.ComponentResolveMetadata;
+import org.gradle.internal.component.model.ConfigurationMetadata;
 import org.gradle.internal.component.model.ModuleSource;
-import org.gradle.internal.exceptions.DefaultMultiCauseException;
 import org.gradle.internal.resolve.ArtifactNotFoundException;
 import org.gradle.internal.resolve.ArtifactResolveException;
 import org.gradle.internal.resolve.ModuleVersionResolveException;
@@ -46,10 +45,7 @@ import org.gradle.internal.resolve.result.BuildableComponentArtifactsResolveResu
 import org.gradle.internal.resolve.result.BuildableModuleComponentMetaDataResolveResult;
 import org.gradle.internal.resolve.result.BuildableModuleVersionListingResolveResult;
 import org.gradle.internal.resolve.result.ErroringResolveResult;
-import org.gradle.internal.resource.transport.http.HttpErrorStatusCodeException;
 
-import java.net.SocketTimeoutException;
-import java.util.List;
 import java.util.Map;
 import java.util.concurrent.Callable;
 
@@ -145,11 +141,10 @@ public class ErrorHandlingModuleComponentRepository implements ModuleComponentRe
         public void listModuleVersions(ModuleDependencyMetadata dependency, BuildableModuleVersionListingResolveResult result) {
             performOperationWithRetries(result,
                     () -> delegate.listModuleVersions(dependency, result),
-                    () -> new ModuleVersionResolveException(dependency.getSelector(), BLACKLISTED_REPOSITORY_ERROR_MESSAGE),
+                    () -> new ModuleVersionResolveException(dependency.getSelector(), () -> BLACKLISTED_REPOSITORY_ERROR_MESSAGE),
                     throwable -> {
                         ModuleComponentSelector selector = dependency.getSelector();
-                        String message = "Failed to list versions for " + selector.getGroup() + ":" + selector.getModule() + ".";
-                        return new ModuleVersionResolveException(selector, message, throwable);
+                        return new ModuleVersionResolveException(selector, () -> "Failed to list versions for " + selector.getGroup() + ":" + selector.getModule() + ".", throwable);
                     });
         }
 
@@ -157,7 +152,7 @@ public class ErrorHandlingModuleComponentRepository implements ModuleComponentRe
         public void resolveComponentMetaData(ModuleComponentIdentifier moduleComponentIdentifier, ComponentOverrideMetadata requestMetaData, BuildableModuleComponentMetaDataResolveResult result) {
             performOperationWithRetries(result,
                     () -> delegate.resolveComponentMetaData(moduleComponentIdentifier, requestMetaData, result),
-                    () -> new ModuleVersionResolveException(moduleComponentIdentifier, BLACKLISTED_REPOSITORY_ERROR_MESSAGE),
+                    () -> new ModuleVersionResolveException(moduleComponentIdentifier, () -> BLACKLISTED_REPOSITORY_ERROR_MESSAGE),
                     throwable -> new ModuleVersionResolveException(moduleComponentIdentifier, throwable)
             );
         }
@@ -172,9 +167,9 @@ public class ErrorHandlingModuleComponentRepository implements ModuleComponentRe
         }
 
         @Override
-        public void resolveArtifacts(ComponentResolveMetadata component, BuildableComponentArtifactsResolveResult result) {
+        public void resolveArtifacts(ComponentResolveMetadata component, ConfigurationMetadata variant, BuildableComponentArtifactsResolveResult result) {
             performOperationWithRetries(result,
-                    () -> delegate.resolveArtifacts(component, result),
+                    () -> delegate.resolveArtifacts(component, variant, result),
                     () -> new ArtifactResolveException(component.getId(), BLACKLISTED_REPOSITORY_ERROR_MESSAGE),
                     throwable -> new ArtifactResolveException(component.getId(), throwable));
         }
@@ -250,7 +245,7 @@ public class ErrorHandlingModuleComponentRepository implements ModuleComponentRe
                     unexpectedFailure = throwable;
                     failure = onError.transform(throwable);
                 }
-                boolean doNotRetry = !isLikelyTransientNetworkingIssue(failure);
+                boolean doNotRetry = !NetworkingIssueVerifier.isLikelyTransientNetworkingIssue(failure);
                 if (doNotRetry || retries == maxTentativesCount) {
                     if (unexpectedFailure != null) {
                         repositoryBlacklister.blacklistRepository(repositoryId, unexpectedFailure);
@@ -269,42 +264,6 @@ public class ErrorHandlingModuleComponentRepository implements ModuleComponentRe
             }
         }
 
-        /**
-         * Determines if an error should cause a retry. We will currently retry:
-         * <ul>
-         * <li>on a network timeout</li>
-         * <li>on a server error (return code 5xx)</li>
-         * <li>on rate limiting</li>
-         * </ul>
-         */
-        private static <E extends Throwable> boolean isLikelyTransientNetworkingIssue(E failure) {
-            if (failure instanceof SocketTimeoutException || failure instanceof HttpHostConnectException) {
-                return true;
-            }
-            if (failure instanceof DefaultMultiCauseException) {
-                List<? extends Throwable> causes = ((DefaultMultiCauseException) failure).getCauses();
-                for (Throwable cause : causes) {
-                    if (isLikelyTransientNetworkingIssue(cause)) {
-                        return true;
-                    }
-                }
-            }
-            if (failure instanceof HttpErrorStatusCodeException) {
-                HttpErrorStatusCodeException httpError = (HttpErrorStatusCodeException) failure;
-                return httpError.isServerError() || isTransientClientError(httpError.getStatusCode());
-            }
-            Throwable cause = failure.getCause();
-            if (cause != null && cause != failure) {
-                return isLikelyTransientNetworkingIssue(cause);
-            }
-            return false;
-        }
-
-        private static boolean isTransientClientError(int statusCode) {
-            return statusCode == HttpStatus.SC_REQUEST_TIMEOUT ||
-                    statusCode == 429; // Too many requests (not available through HttpStatus.XXX)
-        }
-
         @Override
         public MetadataFetchingCost estimateMetadataFetchingCost(ModuleComponentIdentifier moduleComponentIdentifier) {
             return delegate.estimateMetadataFetchingCost(moduleComponentIdentifier);
