@@ -18,6 +18,7 @@ package org.gradle.plugins.ear.internal;
 import groovy.lang.Closure;
 import org.gradle.api.Action;
 import org.gradle.api.model.ObjectFactory;
+import org.gradle.api.provider.Property;
 import org.gradle.api.reflect.HasPublicType;
 import org.gradle.api.reflect.TypeOf;
 import org.gradle.plugins.ear.EarPluginConvention;
@@ -36,6 +37,7 @@ public class DefaultEarPluginConvention extends EarPluginConvention implements H
     private DeploymentDescriptor deploymentDescriptor;
     private String appDirName;
     private String libDirName;
+    private final Property<Boolean> generateDeploymentDescriptor;
 
     @Inject
     public DefaultEarPluginConvention(ObjectFactory objectFactory) {
@@ -43,6 +45,8 @@ public class DefaultEarPluginConvention extends EarPluginConvention implements H
         deploymentDescriptor = objectFactory.newInstance(DefaultDeploymentDescriptor.class);
         deploymentDescriptor.readFrom("META-INF/application.xml");
         deploymentDescriptor.readFrom(appDirName + "/META-INF/" + deploymentDescriptor.getFileName());
+        generateDeploymentDescriptor = objectFactory.property(Boolean.class);
+        generateDeploymentDescriptor.convention(true);
     }
 
     @Override
@@ -83,6 +87,11 @@ public class DefaultEarPluginConvention extends EarPluginConvention implements H
         this.libDirName = libDirName;
     }
 
+    @Override
+    public Property<Boolean> getGenerateDeploymentDescriptor() {
+        return generateDeploymentDescriptor;
+    }
+
     @Override
     public DeploymentDescriptor getDeploymentDescriptor() {
         return deploymentDescriptor;
