@@ -38,7 +38,7 @@ public abstract class AbstractIntegrationTest {
     public final TestNameTestDirectoryProvider testDirectoryProvider = new TestNameTestDirectoryProvider();
 
     public final GradleDistribution distribution = new UnderDevelopmentGradleDistribution(getBuildContext());
-    public final GradleContextualExecuter executer = new GradleContextualExecuter(distribution, testDirectoryProvider, getBuildContext());
+    public final GradleContextualExecuter executer = createExecuter();
 
     public IntegrationTestBuildContext getBuildContext() {
         return IntegrationTestBuildContext.INSTANCE;
@@ -55,6 +55,10 @@ public abstract class AbstractIntegrationTest {
         executer.cleanup();
     }
 
+    protected GradleContextualExecuter createExecuter() {
+        return new GradleContextualExecuter(distribution, testDirectoryProvider, getBuildContext());
+    }
+
     protected GradleDistribution getDistribution() {
         return distribution;
     }
@@ -136,4 +140,8 @@ public abstract class AbstractIntegrationTest {
         action.execute(executer);
         return executer;
     }
+
+    public static String mavenCentralRepository() {
+        return RepoScriptBlockUtil.mavenCentralRepository();
+    }
 }
