@@ -110,8 +110,20 @@ public abstract class DelegatingMavenModule<T extends MavenModule> implements Ma
     }
 
     @Override
-    public MavenModule withGradleMetadataRedirection() {
-        backingModule.withGradleMetadataRedirection();
+    public MavenModule withoutGradleMetadataRedirection() {
+        backingModule.withoutGradleMetadataRedirection();
+        return t();
+    }
+
+    @Override
+    public MavenModule withoutExtraChecksums() {
+        backingModule.withoutExtraChecksums();
+        return t();
+    }
+
+    @Override
+    public MavenModule withExtraChecksums() {
+        backingModule.withoutGradleMetadataRedirection();
         return t();
     }
 
@@ -170,6 +182,7 @@ public abstract class DelegatingMavenModule<T extends MavenModule> implements Ma
         return backingModule.getRootMetaData();
     }
 
+    @Override
     public ModuleArtifact getSnapshotMetaData() {
         return backingModule.getSnapshotMetaData();
     }
@@ -282,12 +295,19 @@ public abstract class DelegatingMavenModule<T extends MavenModule> implements Ma
     }
 
     @Override
-    public void withVariant(String name, Closure<?> action) {
+    public MavenModule withVariant(String name, Closure<?> action) {
         backingModule.withVariant(name, action);
+        return this;
     }
 
     @Override
     public Map<String, String> getAttributes() {
         return backingModule.getAttributes();
     }
+
+    @Override
+    public T withoutDefaultVariants() {
+        backingModule.withoutDefaultVariants();
+        return t();
+    }
 }
