@@ -17,7 +17,6 @@
 package org.gradle.api.publish.ivy;
 
 import org.gradle.api.Action;
-import org.gradle.api.Incubating;
 import org.gradle.api.XmlProvider;
 import org.gradle.internal.HasInternalProtocol;
 
@@ -109,7 +108,6 @@ public interface IvyModuleDescriptorSpec {
      *
      * @since 4.8
      */
-    @Incubating
     void license(Action<? super IvyModuleDescriptorLicense> action);
 
     /**
@@ -117,7 +115,6 @@ public interface IvyModuleDescriptorSpec {
      *
      * @since 4.8
      */
-    @Incubating
     void author(Action<? super IvyModuleDescriptorAuthor> action);
 
     /**
@@ -125,7 +122,6 @@ public interface IvyModuleDescriptorSpec {
      *
      * @since 4.8
      */
-    @Incubating
     void description(Action<? super IvyModuleDescriptorDescription> action);
 
 }
