@@ -33,6 +33,7 @@ import org.gradle.api.reporting.ReportingExtension;
 import org.gradle.plugins.javascript.base.JavaScriptExtension;
 import org.gradle.plugins.javascript.rhino.RhinoExtension;
 import org.gradle.plugins.javascript.rhino.RhinoPlugin;
+import org.gradle.util.DeprecationLogger;
 
 import java.io.File;
 import java.util.concurrent.Callable;
@@ -40,6 +41,7 @@ import java.util.concurrent.Callable;
 public class JsHintPlugin implements Plugin<Project> {
     @Override
     public void apply(Project project) {
+        DeprecationLogger.nagUserOfDeprecatedPlugin("org.gradle.jshint");
         project.getPluginManager().apply(RhinoPlugin.class);
         project.getPluginManager().apply(ReportingBasePlugin.class);
 
