@@ -18,19 +18,21 @@ package org.gradle.launcher.daemon.server.exec;
 import org.gradle.api.logging.LogLevel;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
+import org.gradle.internal.logging.LoggingOutputInternal;
+import org.gradle.internal.logging.events.OutputEvent;
+import org.gradle.internal.logging.events.OutputEventListener;
+import org.gradle.internal.logging.events.ProgressCompleteEvent;
+import org.gradle.internal.logging.events.ProgressEvent;
+import org.gradle.internal.logging.events.ProgressStartEvent;
 import org.gradle.launcher.daemon.diagnostics.DaemonDiagnostics;
 import org.gradle.launcher.daemon.logging.DaemonMessages;
 import org.gradle.launcher.daemon.protocol.Build;
 import org.gradle.launcher.daemon.server.api.DaemonCommandExecution;
 import org.gradle.launcher.daemon.server.api.DaemonConnection;
-import org.gradle.internal.logging.LoggingOutputInternal;
-import org.gradle.internal.logging.events.OutputEvent;
-import org.gradle.internal.logging.events.OutputEventListener;
 
-import java.util.concurrent.BlockingQueue;
+import java.util.Queue;
+import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.CountDownLatch;
-import java.util.concurrent.LinkedBlockingDeque;
-import java.util.concurrent.TimeUnit;
 
 public class LogToClient extends BuildCommandOnly {
 
@@ -47,6 +49,7 @@ public class LogToClient extends BuildCommandOnly {
         this.diagnostics = diagnostics;
     }
 
+    @Override
     protected void doBuild(final DaemonCommandExecution execution, Build build) {
         if (Boolean.getBoolean(DISABLE_OUTPUT)) {
             execution.proceed();
@@ -65,7 +68,7 @@ public class LogToClient extends BuildCommandOnly {
 
     private class AsynchronousLogDispatcher extends Thread {
         private final CountDownLatch completionLock = new CountDownLatch(1);
-        private final BlockingQueue<OutputEvent> eventQueue = new LinkedBlockingDeque<OutputEvent>();
+        private final Queue<OutputEvent> eventQueue = new ConcurrentLinkedQueue<OutputEvent>();
         private final DaemonConnection connection;
         private final OutputEventListener listener;
         private volatile boolean shouldStop;
@@ -75,11 +78,20 @@ public class LogToClient extends BuildCommandOnly {
             super("Asynchronous log dispatcher for " + conn);
             this.connection = conn;
             this.listener = new OutputEventListener() {
+                @Override
                 public void onOutput(OutputEvent event) {
-                    if (dispatcher != null && event.getLogLevel() != null && event.getLogLevel().compareTo(buildLogLevel) >= 0) {
+                    if (dispatcher != null && (isMatchingBuildLogLevel(event) || isProgressEvent(event))) {
                         dispatcher.submit(event);
                     }
                 }
+
+                private boolean isProgressEvent(OutputEvent event) {
+                    return event instanceof ProgressStartEvent || event instanceof ProgressEvent || event instanceof ProgressCompleteEvent;
+                }
+
+                private boolean isMatchingBuildLogLevel(OutputEvent event) {
+                    return event.getLogLevel() != null && event.getLogLevel().compareTo(buildLogLevel) >= 0;
+                }
             };
             LOGGER.debug(DaemonMessages.ABOUT_TO_START_RELAYING_LOGS);
             loggingOutput.addOutputEventListener(listener);
@@ -91,18 +103,19 @@ public class LogToClient extends BuildCommandOnly {
 
         @Override
         public void run() {
-            OutputEvent event;
             try {
                 while (!shouldStop) {
-                    // we must not use interrupt() because it would automatically
-                    // close the connection (sending data from an interrupted thread
-                    // automatically closes the connection)
-                    event = eventQueue.poll(10, TimeUnit.MILLISECONDS);
-                    if (event != null) {
+                    OutputEvent event = eventQueue.poll();
+                    if (event == null) {
+                        Thread.sleep(10);
+                    } else {
                         dispatchAsync(event);
                     }
                 }
             } catch (InterruptedException ex) {
+                // we must not use interrupt() because it would automatically
+                // close the connection (sending data from an interrupted thread
+                // automatically closes the connection)
                 shouldStop = true;
             }
             sendRemainingEvents();
@@ -136,7 +149,7 @@ public class LogToClient extends BuildCommandOnly {
             try {
                 completionLock.await();
             } catch (InterruptedException e) {
-                // the caller has been interrupted
+                Thread.currentThread().interrupt();
             }
         }
     }
