@@ -22,6 +22,7 @@ import org.gradle.internal.logging.events.ProgressStartEvent;
 import org.gradle.internal.operations.BuildOperationCategory;
 import org.gradle.internal.operations.BuildOperationDescriptor;
 import org.gradle.internal.operations.BuildOperationIdFactory;
+import org.gradle.internal.operations.BuildOperationRef;
 import org.gradle.internal.operations.CurrentBuildOperationRef;
 import org.gradle.internal.operations.OperationIdentifier;
 import org.gradle.internal.time.Clock;
@@ -72,16 +73,18 @@ public class DefaultProgressLoggerFactory implements ProgressLoggerFactory {
 
         // Make some assumptions about the console output
         if (buildOperationDescriptor.getOperationType().isTopLevelWorkItem()) {
-            logger.setLoggingHeader(buildOperationDescriptor.getProgressDisplayName());
+            logger.loggingHeader = buildOperationDescriptor.getProgressDisplayName();
         }
 
         return logger;
     }
 
+    @Override
     public ProgressLogger newOperation(String loggerCategory) {
         return init(loggerCategory, null);
     }
 
+    @Override
     public ProgressLogger newOperation(Class loggerClass, ProgressLogger parent) {
         return init(loggerClass.toString(), parent);
     }
@@ -93,6 +96,7 @@ public class DefaultProgressLoggerFactory implements ProgressLoggerFactory {
         if (parentOperation != null && !(parentOperation instanceof ProgressLoggerImpl)) {
             throw new IllegalArgumentException("Unexpected parent logger.");
         }
+        BuildOperationRef currentBuildOperation = currentBuildOperationRef.get();
         return new ProgressLoggerImpl(
             (ProgressLoggerImpl) parentOperation,
             new OperationIdentifier(buildOperationIdFactory.nextId()),
@@ -100,8 +104,8 @@ public class DefaultProgressLoggerFactory implements ProgressLoggerFactory {
             progressListener,
             clock,
             false,
-            currentBuildOperationRef.getId(),
-            currentBuildOperationRef.getParentId(),
+            currentBuildOperation != null ? currentBuildOperation.getId() : null,
+            currentBuildOperation != null ? currentBuildOperation.getParentId() : null,
             null
         );
     }
@@ -165,29 +169,6 @@ public class DefaultProgressLoggerFactory implements ProgressLoggerFactory {
             return this;
         }
 
-        @Override
-        public String getShortDescription() {
-            return null;
-        }
-
-        @Override
-        public ProgressLogger setShortDescription(String shortDescription) {
-            assertCanConfigure();
-            return this;
-        }
-
-        @Override
-        public String getLoggingHeader() {
-            return loggingHeader;
-        }
-
-        @Override
-        public ProgressLogger setLoggingHeader(String loggingHeader) {
-            assertCanConfigure();
-            this.loggingHeader = loggingHeader;
-            return this;
-        }
-
         @Override
         public ProgressLogger start(String description, String status) {
             setDescription(description);
@@ -242,19 +223,23 @@ public class DefaultProgressLoggerFactory implements ProgressLoggerFactory {
             ));
         }
 
+        @Override
         public void progress(String status) {
             progress(status, false);
         }
 
+        @Override
         public void progress(String status, boolean failing) {
             assertRunning();
             listener.progress(new ProgressEvent(progressOperationId, ensureNotNull(status), failing));
         }
 
+        @Override
         public void completed() {
             completed(null, false);
         }
 
+        @Override
         public void completed(String status, boolean failed) {
             assertRunning();
             state = State.completed;
