@@ -17,7 +17,6 @@
 package org.gradle.api.publish.maven;
 
 import org.gradle.api.Action;
-import org.gradle.api.Incubating;
 import org.gradle.api.component.SoftwareComponent;
 import org.gradle.api.publish.Publication;
 import org.gradle.api.publish.VersionMappingStrategy;
@@ -63,7 +62,7 @@ import org.gradle.internal.HasInternalProtocol;
  *
  * task sourceJar(type: Jar) {
  *   from sourceSets.main.allJava
- *   classifier "sources"
+ *   archiveClassifier = "sources"
  * }
  *
  * publishing {
@@ -130,7 +129,7 @@ public interface MavenPublication extends Publication {
      *
      * Currently 3 types of component are supported: 'components.java' (added by the JavaPlugin), 'components.web' (added by the WarPlugin)
      * and `components.javaPlatform` (added by the JavaPlatformPlugin).
-     * 
+     *
      * For any individual MavenPublication, only a single component can be provided in this way.
      *
      * The following example demonstrates how to publish the 'java' component to a Maven repository.
@@ -169,7 +168,7 @@ public interface MavenPublication extends Publication {
      * apply plugin: "maven-publish"
      *
      * task sourceJar(type: Jar) {
-     *   classifier "sources"
+     *   archiveClassifier = "sources"
      * }
      *
      * publishing {
@@ -198,7 +197,7 @@ public interface MavenPublication extends Publication {
      * apply plugin: "maven-publish"
      *
      * task sourceJar(type: Jar) {
-     *   classifier "sources"
+     *   archiveClassifier = "sources"
      * }
      *
      * publishing {
@@ -233,7 +232,7 @@ public interface MavenPublication extends Publication {
      * apply plugin: "maven-publish"
      *
      * task sourceJar(type: Jar) {
-     *   classifier "sources"
+     *   archiveClassifier = "sources"
      * }
 
      * publishing {
@@ -312,6 +311,26 @@ public interface MavenPublication extends Publication {
      *
      * @since 5.2
      */
-    @Incubating
     void versionMapping(Action<? super VersionMappingStrategy> configureAction);
+
+    /**
+     * Silences the compatibility warnings for the Maven publication for the specified variant.
+     *
+     * Warnings are emitted when Gradle features are used that cannot be mapped completely to Maven POM.
+     *
+     * @param variantName the variant to silence warning for
+     *
+     * @since 6.0
+     */
+    void suppressPomMetadataWarningsFor(String variantName);
+
+
+    /**
+     * Silences all the compatibility warnings for the Maven publication.
+     *
+     * Warnings are emitted when Gradle features are used that cannot be mapped completely to Maven POM.
+     *
+     * @since 6.0
+     */
+    void suppressAllPomMetadataWarnings();
 }
