@@ -16,16 +16,20 @@
 
 package org.gradle.api.internal.provider;
 
-import org.gradle.api.internal.tasks.TaskDependencyResolveContext;
+import org.gradle.internal.DisplayName;
 
 import java.util.Collection;
+import java.util.List;
 import java.util.Map;
 
-public interface MapCollector<K, V> {
+/**
+ * A supplier of zero or more mappings from value of type {@link K} to value of type {@link V}.
+ */
+public interface MapCollector<K, V> extends ValueSupplier {
 
     boolean present();
 
-    void collectInto(MapEntryCollector<K, V> collector, Map<K, V> dest);
+    void collectInto(DisplayName owner, MapEntryCollector<K, V> collector, Map<K, V> dest);
 
     boolean maybeCollectInto(MapEntryCollector<K, V> collector, Map<K, V> dest);
 
@@ -33,5 +37,5 @@ public interface MapCollector<K, V> {
 
     boolean maybeCollectKeysInto(ValueCollector<K> collector, Collection<K> dest);
 
-    boolean maybeVisitBuildDependencies(TaskDependencyResolveContext context);
+    void visit(List<ProviderInternal<? extends Map<? extends K, ? extends V>>> sources);
 }
