@@ -18,6 +18,7 @@ package org.gradle.internal.instantiation;
 
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.service.ServiceLookup;
+import org.gradle.internal.state.ManagedFactory;
 
 import java.lang.annotation.Annotation;
 import java.util.Collection;
@@ -35,14 +36,14 @@ public interface InstantiatorFactory {
      *
      * @param services The services to make available to instances.
      */
-    Instantiator inject(ServiceLookup services);
+    InstanceGenerator inject(ServiceLookup services);
 
     /**
      * Creates an {@link Instantiator} that can inject user provided values into the instances it creates, but does not decorate the instances. Is not lenient.
      *
      * <p>Use for any non-model types for which user provided values, but no services, need to be injected. This is simply a convenience for {@link #injectScheme()}.
      */
-    Instantiator inject();
+    InstanceGenerator inject();
 
     /**
      * Create an {@link InstantiationScheme} that can inject services and user provided values into the instances it creates, but does not decorate the instances. Supports using the {@link javax.inject.Inject} annotation only. Is not lenient.
@@ -67,7 +68,7 @@ public interface InstantiatorFactory {
      * where backwards compatibility is required and instead prefer {@link #inject(ServiceLookup)} for any new non DSL-types.
      * This method will be retired in the future.
      */
-    Instantiator injectLenient(ServiceLookup services);
+    InstanceGenerator injectLenient(ServiceLookup services);
 
     /**
      * Creates an {@link Instantiator} that can inject user provided values into the instances it creates, but does not decorate the instances.
@@ -78,7 +79,7 @@ public interface InstantiatorFactory {
      * where backwards compatibility is required and instead prefer {@link #inject()} for any new non DSL-types.
      * This method will be retired in the future.
      */
-    Instantiator injectLenient();
+    InstanceGenerator injectLenient();
 
     /**
      * Creates an {@link Instantiator} that can inject services and user provided values into the instances it creates and also decorates the instances.
@@ -87,7 +88,7 @@ public interface InstantiatorFactory {
      *
      * @param services The services to make available to instances.
      */
-    Instantiator injectAndDecorate(ServiceLookup services);
+    InstanceGenerator injectAndDecorate(ServiceLookup services);
 
     /**
      * Create an {@link InstantiationScheme} that can inject services and user provided values into the instances it creates and also decorates the instances. Supports using the {@link javax.inject.Inject} annotation only. Is not lenient.
@@ -99,7 +100,7 @@ public interface InstantiatorFactory {
      *
      * <p>Use for any model types for which no user provided constructor values or services need to be injected. This is a convenience for {@link #injectAndDecorateLenient(ServiceLookup)} and will also be retired.
      */
-    Instantiator decorateLenient();
+    InstanceGenerator decorateLenient();
 
     /**
      * Creates an {@link Instantiator} that can inject services and user provided values into the instances it creates and also decorates the instances.
@@ -112,5 +113,10 @@ public interface InstantiatorFactory {
      *
      * @param services The registry of services to make available to instances.
      */
-    Instantiator injectAndDecorateLenient(ServiceLookup services);
+    InstanceGenerator injectAndDecorateLenient(ServiceLookup services);
+
+    /**
+     * Returns a managed factory to use when isolating managed objects created using this factory.
+     */
+    ManagedFactory getManagedFactory();
 }
