@@ -112,11 +112,26 @@ public abstract class DefaultSourceSet implements SourceSet {
         return getTaskName("process", "resources");
     }
 
+    @Override
+    public String getJavadocTaskName() {
+        return getTaskName(null, JavaPlugin.JAVADOC_TASK_NAME);
+    }
+
     @Override
     public String getJarTaskName() {
         return getTaskName(null, "jar");
     }
 
+    @Override
+    public String getJavadocJarTaskName() {
+        return getTaskName(null, "javadocJar");
+    }
+
+    @Override
+    public String getSourcesJarTaskName() {
+        return getTaskName(null, "sourcesJar");
+    }
+
     @Override
     public String getTaskName(@Nullable String verb, @Nullable String target) {
         return namingScheme.getTaskName(verb, target);
@@ -185,6 +200,16 @@ public abstract class DefaultSourceSet implements SourceSet {
         return configurationNameOf(JavaPlugin.RUNTIME_ELEMENTS_CONFIGURATION_NAME);
     }
 
+    @Override
+    public String getJavadocElementsConfigurationName() {
+        return configurationNameOf(JavaPlugin.JAVADOC_ELEMENTS_CONFIGURATION_NAME);
+    }
+
+    @Override
+    public String getSourcesElementsConfigurationName() {
+        return configurationNameOf(JavaPlugin.SOURCES_ELEMENTS_CONFIGURATION_NAME);
+    }
+
     @Override
     public SourceSetOutput getOutput() {
         return output;
