@@ -16,8 +16,6 @@
 
 package org.gradle.internal.snapshot;
 
-import org.gradle.api.UncheckedIOException;
-
 import javax.annotation.Nullable;
 
 public interface ValueSnapshotter {
@@ -26,12 +24,14 @@ public interface ValueSnapshotter {
      *
      * <p>The snapshots must contain no references to the ClassLoader of the value.</p>
      *
-     * @throws UncheckedIOException On failure to snapshot the value.
+     * @throws ValueSnapshottingException On failure to snapshot the value.
      */
-    ValueSnapshot snapshot(@Nullable Object value) throws UncheckedIOException;
+    ValueSnapshot snapshot(@Nullable Object value) throws ValueSnapshottingException;
 
     /**
      * Creates a snapshot of the given value, given a candidate snapshot. If the value is the same as the value provided by the candidate snapshot, the candidate <em>must</em> be returned.
+     *
+     * @throws ValueSnapshottingException On failure to snapshot the value.
      */
-    ValueSnapshot snapshot(@Nullable Object value, ValueSnapshot candidate);
+    ValueSnapshot snapshot(@Nullable Object value, ValueSnapshot candidate) throws ValueSnapshottingException;
 }
