@@ -18,6 +18,7 @@ package org.gradle.tooling.internal.provider.runner;
 
 import org.gradle.initialization.BuildEventConsumer;
 import org.gradle.internal.operations.BuildOperationDescriptor;
+import org.gradle.internal.operations.BuildOperationIdFactory;
 import org.gradle.internal.operations.BuildOperationListener;
 import org.gradle.internal.operations.OperationFinishEvent;
 import org.gradle.internal.operations.OperationIdentifier;
@@ -36,9 +37,11 @@ import static java.util.Collections.emptyList;
 public class ToolingApiSubscribableBuildActionRunnerRegistration implements SubscribableBuildActionRunnerRegistration {
 
     private final OperationResultPostProcessor operationResultPostProcessor;
+    private final BuildOperationIdFactory idFactory;
 
-    ToolingApiSubscribableBuildActionRunnerRegistration(OperationResultPostProcessor operationResultPostProcessor) {
-        this.operationResultPostProcessor = operationResultPostProcessor;
+    ToolingApiSubscribableBuildActionRunnerRegistration(BuildOperationIdFactory idFactory, CompositeOperationResultPostProcessor compositeOperationResultPostProcessor) {
+        this.idFactory = idFactory;
+        this.operationResultPostProcessor = compositeOperationResultPostProcessor;
     }
 
     @Override
@@ -51,7 +54,11 @@ public class ToolingApiSubscribableBuildActionRunnerRegistration implements Subs
         List<Object> listeners = new ArrayList<Object>();
         listeners.add(parentTracker);
         if (clientSubscriptions.isRequested(OperationType.TEST)) {
-            listeners.add(new ClientForwardingTestOperationListener(progressEventConsumer, clientSubscriptions));
+            BuildOperationListener buildListener = new ClientForwardingTestOperationListener(progressEventConsumer, clientSubscriptions);
+            if (clientSubscriptions.isRequested(OperationType.TEST_OUTPUT)) {
+                buildListener = new ClientForwardingTestOutputOperationListener(buildListener, progressEventConsumer, idFactory);
+            }
+            listeners.add(buildListener);
         }
         if (clientSubscriptions.isAnyRequested(OperationType.GENERIC, OperationType.WORK_ITEM, OperationType.TASK, OperationType.PROJECT_CONFIGURATION, OperationType.TRANSFORM)) {
             BuildOperationListener buildListener = NO_OP;
