@@ -16,13 +16,14 @@
 
 package org.gradle.workers.internal;
 
+import org.gradle.internal.instantiation.InstantiatorFactory;
 import org.gradle.internal.service.ServiceRegistry;
 
 public class FlatClassLoaderWorker extends AbstractClassLoaderWorker {
     private final ClassLoader workerClassLoader;
 
-    public FlatClassLoaderWorker(ClassLoader workerClassLoader, ServiceRegistry serviceRegistry) {
-        super(serviceRegistry);
+    public FlatClassLoaderWorker(ClassLoader workerClassLoader, ServiceRegistry serviceRegistry, ActionExecutionSpecFactory actionExecutionSpecFactory, InstantiatorFactory instantiatorFactory) {
+        super(serviceRegistry, actionExecutionSpecFactory, instantiatorFactory);
         this.workerClassLoader = workerClassLoader;
     }
 
